# Copyright 2021 Element Analytics, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Contains methods to interact with graphs api
"""
import json
import io

import json_lines

from unify.properties import Properties
from unify.properties import ClusterSetting
from unify.apirequestsmng import ApiRequestManager
from unify.helpers.SingleOrg import single_org


class Users(ApiRequestManager):
    """
    Class to interact with pipeline endpoints
    """

    def __init__(self, cluster=None, props=Properties(ClusterSetting.KEY_RING), org_id=None):
        """
        Class constructor

        :param cluster: Cluster name to be used
        :type cluster: str
        :param props: Properties instantiated object
        :type props: class:`unify.properties.Properties`
        """
        super().__init__(cluster=cluster, props=props, org_id=org_id)

        self.session.headers.update(
            {
                self.x_auth_token_header: self.props.get_auth_token(cluster=self.cluster)
            }
        )

        remote = self.props.get_remote(self.cluster)

        self.home_url = remote + 'api/v1/users'

        self.users_url = self.home_url + '/{}'

        self.password_url = self.users_url + '/password'

    @single_org
    def change_user_password(self, org_id=None, *, user_id, old_password, new_password):
        body = {"newPassword": new_password, "oldPassword": old_password}

        if org_id is not None:
            headers = self.build_header(
                org_id=org_id,
                others=self.content_type_header
            )
        else:
            headers = self.build_header(
                others=self.content_type_header
            )

        request = self.session.put(
            self.password_url.format(user_id),
            headers=headers,
            json=body
        )

        if request.status_code in self.OK:
            return json.loads(request.content)

        raise Exception(request.content.decode())

    @single_org
    def get_users_list(self, org_id=None, *, user_ids: list):

        data = tuple(["userId", user_id] for user_id in user_ids)

        if org_id is not None:
            headers = self.build_header(
                org_id=org_id,
                others=self.content_type_header
            )
        else:
            headers = self.build_header(
                others=self.content_type_header
            )

        request = self.session.get(
            self.home_url,
            headers=headers,
            params=data
        )

        if request.status_code in self.OK:
            return json.loads(request.content)

        raise Exception(request.content.decode())

    @single_org
    def delete_user(self, org_id=None, *, user_id):

        if org_id is not None:
            headers = self.build_header(
                org_id=org_id,
                others=self.content_type_header
            )
        else:
            headers = self.build_header(
                others=self.content_type_header
            )

        request = self.session.delete(
            self.users_url.format(user_id),
            headers=headers,
        )

        if request.status_code in self.OK:
            return json.loads(request.content)

        raise Exception(request.content.decode())



# Copyright 2021 Element Analytics, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Contains methods to interact with templates api
"""
import json
import os

from unify.generalutils import stream_iterable
from unify.properties import Properties
from unify.properties import ClusterSetting
from unify.apirequestsmng import ApiRequestManager
from unify.generalutils import csv_to_json, json_to_csv
from unify.helpers.SingleOrg import single_org
from unify.helpers.unify_objects.Template import Template as TemplateData


class Templates(ApiRequestManager):
    """
    Template client class to interact with templates endpoints
    """

    def __init__(self, cluster=None, props=Properties(ClusterSetting.KEY_RING), org_id=None):
        """
        Template constructor

        :param cluster: Cluster name to interact with
        :type cluster: str
        :param props: Instantiated Properties class
        :type props: class:`unify.properties.Properties`
        """
        super().__init__(cluster=cluster, props=props, org_id=org_id)

        try:

            self.list_templates_uri = 'api/assetTemplates'

            self.upload_content_type = 'text/tab-separated-values'

            self.template_mng = "api/template_management/v1/orgs/{}"

            self.org_uom_url = self.props.get_remote(self.cluster) + self.template_mng + "/uoms"

            self.template_list_url = "api/template_management/v1/orgs/{}/templates"

            self.delete_template_uri = 'api/template_management/v1/templates/{}'

            self.root_templete_url = self.props.get_remote(
                self.cluster) + self.list_templates_uri

            self.upload_endpoint = self.props.get_remote(
                self.cluster) + 'api/template_management/v1/templates/upload'

            self.upload_parameters_endpoint = self.props.get_remote(
                self.cluster) + 'template_management/v1/orgs/{}/templates/metadata/upload'

            self.get_sensors_uri = self.root_templete_url + "/{}/sensors"

            self.template_download_url = self.props.get_remote(
                self.cluster) + "api/template_management/v1/templates/download"

            self.template_batch_url = self.props.get_remote(
                self.cluster) + 'api/template_management/v1/orgs/{}/templates/batch'

            self.template_sensor_coverage_uri = self.props.get_remote(
                self.cluster) + "api/template_management/v1/templates/sensor-coverage"

            self.template_url = self.props.get_remote(
                self.cluster) + 'api/template_management/v1/templates'

            self.template_attribute = self.props.get_remote(
                self.cluster) + 'api/template_management/v1/orgs/{}/templates/{}/attributes'

            self.template_attribute_url = self.props.get_remote(
                self.cluster) + 'api/template_management/v1/orgs/{}/templates/{}/attributes/{}'

            self.categories_url = self.props.get_remote(
                self.cluster) + 'api/template_management/v1/orgs/{}/categories'

        except Exception as error:
            raise error

    @single_org
    def upload_template(self, file_path, org_id=None):
        """
        Uploads the given file path with templates data.

        :param file_path: File directory path with the templates contents
        :type file_path: str
        :param org_id: Org id where templates are going to be saved
        :type org_id: int or str
        :return: Upload template status message
        """

        if os.path.exists(file_path):
            file_upload = {'file': open(file_path, 'r+')}
        else:
            file_upload = {'file': file_path}

        upload_headers = self.build_header(
            org_id=org_id,
            aut_token=self.props.get_auth_token(cluster=self.cluster)
        )

        post_upload_template = self.session.post(
            self.upload_endpoint,
            headers=upload_headers,
            files=file_upload
        )

        if post_upload_template.status_code == 201:
            return json.loads(post_upload_template.content)

        raise Exception(post_upload_template.content)

    @single_org
    def get_attributes(self, org_id=None, template_id=None):
        """
        Downloads all the attributes of the given template id.

        :param org_id: Org where the templates are stored
        :type org_id: int or str
        :param template_id: Template id which attributes are being downloaded
        :type template_id: int or str
        :return: Retrieve attributes status message
        """
        final_url = self.template_attribute.format(org_id, template_id)

        upload_headers = self.build_header(
            org_id=org_id,
            aut_token=self.props.get_auth_token(cluster=self.cluster)
        )

        get_attrs = self.session.get(
            final_url,
            headers=upload_headers,
            params=[("pageSize", 2000)]
        )

        if get_attrs.status_code == 200:
            return json.loads(get_attrs.content)

        raise Exception(repr(get_attrs.content))

    @single_org
    def upload_string_content_file(self, org_id=None, content=None):
        """
        Uploads templates data that are stored in the variable content.

        :param org_id: Org id where templates are going to be stored
        :type org_id: int or str
        :param content: Templates contents
        :type content: str
        :return: Upload template status message
        """

        files = {'file': ('templates_uploads.csv', content, self.upload_content_type)}

        upload_headers = self.build_header(
            org_id=org_id,
            aut_token=self.props.get_auth_token(cluster=self.cluster)
        )

        post_upload_template = self.session.post(
            self.upload_endpoint,
            headers=upload_headers,
            files=files
        )

        if post_upload_template.status_code == 201:
            return json.loads(post_upload_template.content)

        raise Exception(post_upload_template.content)

    @single_org
    def upload_config(self, org_id=None, file_path=None):
        """
        Uploads template configuration file.

        :param org_id: Org where the config will be applied
        :type org_id: int or str
        :param file_path: Directory file path where the contest are
        :type file_path: str
        :return: Upload template configuration status message
        """
        try:

            file_reader = open(file_path, "r+")
            try:
                content = file_reader.read()

            except Exception as e:
                file_reader.close()
                raise Exception(str(e))
            finally:
                file_reader.close()
        except FileNotFoundError:
            raise FileNotFoundError("File {} does not exist".format(file_path))

        return self.upload_config_with_content(org_id, content)

    @single_org
    def upload_config_with_content(self, org_id=None, content=None, format="csv"):
        """
        Uploads template configuration from content of a variable and not a file.

        :param org_id: Org where the templates are stored
        :type org_id: int or str
        :param content: Template config content
        :type content: str
        :param format: CSV or TSV
        :type format: str, optional
        :return: Upload template attribute configuration message
        """

        files = {
            'file': (
                f'templates_uploads.{format}',
                content,
                self.upload_content_type
            )
        }

        upload_headers = self.build_header(
            org_id=org_id,
            aut_token=self.props.get_auth_token(cluster=self.cluster)
        )

        post_upload_template = self.session.post(
            self.upload_parameters_endpoint.format(org_id),
            headers=upload_headers,
            files=files
        )

        if post_upload_template.status_code == 200:
            return json.loads(post_upload_template.content)

        raise Exception(post_upload_template.content)

    @single_org
    def _add_category(self, org_id=None, category=None):
        url = self.categories_url.format(org_id)
        headers = self.build_header(
            org_id=org_id,
            aut_token=self.props.get_auth_token(cluster=self.cluster),
            others=self.content_type_header
        )
        payload = {
            "label": category,
        }

        request = self.session.post(
            url,
            headers=headers,
            data=json.dumps(payload),
        )
        if request.status_code == 200:
            return json.loads(request.content)
        raise Exception(request.content)

    @single_org
    def list_all_categories(self, org_id=None):
        """
        Retrieves all the template categories on the given org.

        :param org_id: Org id to be queried
        :type org_id: int or str
        :return: List of template categories
        """
        headers = self.build_header(
            org_id=org_id,
            aut_token=self.props.get_auth_token(cluster=self.cluster)
        )
        request = self.session.get(
            self.categories_url.format(org_id),
            headers=headers
        )
        if request.status_code == 200:
            entries = json.loads(request.content)['entries']
            entries_dic = {}
            for entry in entries:
                label = entry['label']
                entries_dic.update({label: entry['id']})
            return entries_dic
        raise Exception(request.content)

    @single_org
    def category(self, org_id=None, *, template_id, template_name, version, categories):
        """
        Updates template categories

        :param org_id: Org where the templates are stored
        :type org_id: int or str
        :param template_id: Template id
        :type template_id: int or str
        :param template_name: Template name
        :type template_name: str
        :param version: Template version
        :type version: int
        :param categories: List contains the templates categories
        :type categories: list or str
        :return: Update categories status message
        """
        existing = self.list_all_categories(org_id)
        ids = []
        for cat in categories:
            if cat in existing:
                ids.append(existing.get(cat))
            else:
                _id = self._add_category(org_id, cat)['id']
                ids.append(_id)

        url = '{}/{}?version={}'.format(self.template_url, template_id, version)
        headers = self.build_header(
            org_id=org_id,
            aut_token=self.props.get_auth_token(cluster=self.cluster),
            others=self.content_type_header
        )
        payload = {
            "organizationId": org_id,
            "name": template_name,
            "categoryIds": ids
        }

        request = self.session.put(
            url,
            headers=headers,
            data=json.dumps(payload),
            params=[("version", version)]
        )
        if request.status_code == 200:
            return json.loads(request.content)

        raise Exception(request.content)

    @single_org
    def get_template(self, org_id=None, *, template_id):
        """
        Retrieves template by id.

        :param org_id: Org id of whom templates are being retrieved
        :type org_id: int or str
        :param template_id: Template id
        :type template_id: int or str
        :return: Retrieved template str
        """
        header = self.build_header(
            org_id=org_id,
            aut_token=self.props.get_auth_token(cluster=self.cluster)
        )
        url = self.template_url + '/' + str(template_id)
        request = self.session.get(url, headers=header)

        if request.status_code == 200:
            return json.loads(request.content)

        raise Exception(json.loads(request.content))

    @single_org
    def list_asset_templates(self, org_id=None):
        """
        Retrieves the templates list from the given org.

        :param org_id: Org id of whom templates are being retrieved
        :type org_id: int or str
        :return: List of templates
        """

        header = self.build_header(
            org_id=org_id,
            aut_token=self.props.get_auth_token(cluster=self.cluster)
        )

        list_request = self.session.get(self.root_templete_url, headers=header)

        if list_request.status_code == 200:
            return json.loads(list_request.content)["items"]

        raise Exception(json.loads(list_request.content))

    @single_org
    def download_template(self, org_id=None, list_templates=None):
        """
        Downloads the templates data.

        :param org_id: Org where the templates exists
        :type org_id: int or str
        :param list_templates: List containing the template ids
        :type list_templates: list of str
        :return: Text representing templates
        """

        if list_templates is None:
            list_templates = []
        list_templates = list_templates if list_templates is not None else []

        header = self.build_header(
            org_id=org_id,
            aut_token=self.props.get_auth_token(cluster=self.cluster),
            others=self.content_type_header
        )

        data = tuple(["templateId", name] for name in list_templates)

        get_response = self.session.get(self.template_download_url, headers=header, params=data)

        if get_response.status_code == 200:
            return get_response.content

        raise Exception(json.loads(get_response.content))

    @single_org
    def download_all_templates(self, org_id=None, format="csv"):
        """
        Retrieves all the templates data.

        :param org_id: Org where the templates are stored
        :type org_id: int or str
        :param format: File format. Accepts "CSV" or "JSON"
        :type format: str, optional
        :return: Text representing templates
        """
        try:
            template_list_resp = self.list_asset_templates(org_id=org_id)

            template_list = [template["id"] for template in template_list_resp]

            data = self.download_template(org_id=org_id, list_templates=template_list)

            if format == "csv":
                return data

            return json.dumps(csv_to_json(csv_data=data))

        except Exception as error:
            raise error

    @single_org
    def download_template_batches(self, org_id=None, format: str = "csv", batch_size: int = 100):
        """
        Retrieves all the templates data in batches

        :param org_id: Org where the templates are stored
        :type org_id: int or str
        :param format: File format. Accepts "CSV" or "JSON"
        :type format: str, optional
        :param batch_size: The size of each batch
        :type batch_size: integer, optiona
        :return: Text representing templates
        """
        try:
            template_list_resp = self.list_asset_templates(org_id=org_id)

            full_list = []

            for template_chunk in stream_iterable(container=template_list_resp, chunk=batch_size):
                template_list = [template["id"] for template in template_chunk]

                data = self.download_template(org_id=org_id, list_templates=template_list)

                full_list.extend(csv_to_json(csv_data=data))

            if format == "csv":
                return json_to_csv(full_list)

            return json.dumps(full_list)

        except Exception as error:
            raise error

    @single_org
    def download_all_template_config(self, org_id=None):
        """
        Retrieves all template configurations.

        :param org_id: Org where the templates are stored
        :type org_id: int or str
        :return: Text representing template metadata
        """
        try:

            header = self.build_header(
                org_id=org_id,
                aut_token=self.props.get_auth_token(cluster=self.cluster)
            )

            url = self.template_batch_url.format(org_id)

            response = self.session.get(url, headers=header)

            if response.status_code not in [200, 201]:
                raise Exception(response.content)

            config_array = [["template", "attribute", "key", "value"]]

            template_sets = response.json()["templates"]

            for template_set in template_sets:

                # retrieve template configuration
                template_data = {"name": template_set["template"]["name"]}

                template_config_list = template_set["template"]["metadataCollection"]
                for template_config in template_config_list:
                    config_array.append([template_data["name"],
                                         "",
                                         template_config.get("key"),
                                         template_config.get("value")])

                # retrieve template attribute configuration
                for attribute in template_set["attributes"]:

                    attribute_config_list = attribute["metadataCollection"]
                    for attribute_config in attribute_config_list:
                        config_array.append([
                            template_data["name"],
                            attribute["name"],
                            attribute_config.get("key"),
                            attribute_config.get("value")])

            return "\n".join([",".join(row) for row in config_array])

        except Exception as error:
            raise error

    @single_org
    def download_given_templates(self, org_id=None, template_list=None, format="csv"):
        """
        Downloads data from templates.

        :param org_id: Org where the templates exists
        :type org_id: int or str
        :param template_list: List containing the template ids
        :type template_list: list of int
        :param format: File format. Accepts "CSV" or "JSON"
        :type format: str
        :return: Text representing templates
        """
        try:

            list_templates = template_list if template_list is not None else []

            data = self.download_template(org_id=org_id, list_templates=list_templates)

            if format == "csv":
                return data

            return json.dumps(csv_to_json(csv_data=data))

        except Exception as error:
            raise error

    @single_org
    def create_template_attribute_params(self, org_id=None, *, template_id, attribute_name, sanitized_name, data_type,
                                         attribute_type,
                                         description=None, uom=None, interpolation=None):
        """
        Creates a Custom template attribute

        :param org_id: Org where the template exist
        :param template_id: Template id where the new attribute will be added
        :param attribute_name: Attribute full name
        :param sanitized_name: Attribute Sanitized full name
        :param data_type: Attribute Data type
        :param attribute_type: Attribute type
        :param description: Attribute description
        :param uom: Attribute Unit Of Measure
        :param interpolation: Attribute Interpolation Method
        :return:
        """

        new_template = TemplateData()

        for param, value in locals().items():
            new_template.__setattr__(param, value)

        return self.create_template_attribute(org_id=org_id, template=new_template)

    @single_org
    def create_template_attribute(self, org_id=None, *, template: TemplateData):
        """
        Creates a Custom template attribute

        :param org_id:
        :param template:
        :return:
        """

        header = self.build_header(
            org_id=org_id,
            aut_token=self.props.get_auth_token(cluster=self.cluster),
            others=self.content_type_header
        )

        create_attribute = self.session.post(
            self.template_attribute.format(org_id, template.template_id),
            headers=header,
            data=template.to_json()
        )

        if create_attribute.status_code in [200, 201]:
            return json.loads(create_attribute.content)

        raise Exception(json.loads(create_attribute.content))

    @single_org
    def get_uom(self, org_id=None):
        """
        Retrieve the list of unit of measure on the org

        :param org_id: Organization id to be queried
        :return:
        """
        header = self.build_header(
            org_id=org_id,
            aut_token=self.props.get_auth_token(cluster=self.cluster),
            others=self.content_type_header
        )

        get_response = self.session.get(self.org_uom_url.format(org_id), headers=header)

        if get_response.status_code in [200]:
            return json.loads(get_response.content)

        raise Exception(json.loads(get_response.content))



# Copyright 2021 Element Analytics, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Contains methods to interact with graphs api
"""
import json

from unify.properties import Properties
from unify.properties import ClusterSetting
from unify.apirequestsmng import ApiRequestManager
from unify.helpers.SingleOrg import single_org


class Hierarchy(ApiRequestManager):
    """
    Class to interact with hierarchies endpoints
    """

    def __init__(self, cluster=None, props=Properties(ClusterSetting.KEY_RING), org_id=None):
        """
        Class constructor

        :param cluster: Cluster name to be used
        :type cluster: str
        :param props: Properties instantiated object
        :type props: class:`unify.properties.Properties`
        """
        super().__init__(cluster=cluster, props=props, org_id=org_id)

        remote = self.props.get_remote(self.cluster)

        self.home_hierarchies_url = remote + "api/v2/orgs/{}/asset_hierarchies"

        self.home_hierarchies_2_url = remote + "api/v2/asset_hierarchies"

        self.add_level_url = remote + "api/v2/asset_hierarchies/{}/levels"

        self.specific_hierarchy_url = remote + "api/v2/asset_hierarchies/{}"

    @single_org
    def get_all_hierarchies(self, org_id=None):
        """
        retrieve all hierarchies on the given organization

        :param org_id: organization id to be queried
        :return:
        """
        header = self.build_header(
            org_id=org_id,
            aut_token=self.props.get_auth_token(cluster=self.cluster),
            others=self.content_type_header
        )

        result = self.session.get(
            self.home_hierarchies_2_url,
            headers=header
        )

        if result.status_code == 200:
            return json.loads(result.content)

        raise Exception(repr(result.content))

    @single_org
    def create_hierarchy(self, org_id=None, *, name, levels=None, attribute_names=None, private=False):
        """
        Creates a new hierarchy with the given configuration

        :param org_id: Organization id where the hierarchy will be created
        :param name: Name for the new hierarchy, unique
        :param levels: Arrays containing the name of levels
        :param attribute_names:
        :param private:
        :return:
        """

        if levels is None:
            levels = []

        header = self.build_header(
            org_id=org_id,
            aut_token=self.props.get_auth_token(cluster=self.cluster),
            others=self.content_type_header
        )

        body = {
            "name": name,
            "levels": [],
            "assetAttributeNames": attribute_names if attribute_names is not None else [],
            "isPrivate": private,
            "draft": False
        }
        result = self.session.post(
            self.home_hierarchies_2_url,
            headers=header,
            data=json.dumps(body)
        )

        if result.status_code in [201]:
            content = json.loads(result.content)
            for level in levels:
                self.add_level(
                    org_id=org_id,
                    hierarchy_id=content["id"],
                    level_name=level
                )

            return content

        raise Exception(repr(result.content))

    @single_org
    def get_hierarchy(self, org_id=None, *, hierarchy_id):
        """
        Retrieves all the information of the given hierarchy

        :param org_id: organization id where the hierarchy is stored
        :param hierarchy_id:
        :return:
        """
        header = self.build_header(
            org_id=org_id,
            aut_token=self.props.get_auth_token(cluster=self.cluster),
            others=self.content_type_header
        )

        result = self.session.get(
            self.specific_hierarchy_url.format(hierarchy_id),
            headers=header
        )

        if result.status_code == 200:
            return json.loads(result.content)

        raise Exception(repr(result.content))

    @single_org
    def add_level(self, org_id=None, *, hierarchy_id, level_name):
        """
        Add a node level to the given hierarchy

        :param org_id: organization id where the hierarchy is stored
        :param hierarchy_id:
        :param level_name:
        :return:
        """

        header = self.build_header(
            org_id=org_id,
            aut_token=self.props.get_auth_token(cluster=self.cluster),
            others=self.content_type_header
        )

        body = {
            "groupName": level_name
        }

        result = self.session.post(
            self.add_level_url.format(hierarchy_id),
            headers=header,
            data=json.dumps(body)
        )

        if result.status_code in [201]:
            return json.loads(result.content)

        raise Exception(repr(result.content))



# Copyright 2021 Element Analytics, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
import json

from tests import *
from unify.hierarchies import Hierarchy


class HierrachiesDefaultOrg(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.hierarchy_mng = Hierarchy(
            cluster=cluster_name,
            props=props,
            org_id=test_org
        )

    def test_get_all_hierarchies_display(self):
        export = self.hierarchy_mng.get_all_hierarchies()

        self.assertTrue(len(export) > 0, export)

    def test_create_hierarchy(self):
        export = self.hierarchy_mng.create_hierarchy(
            name=str(uuid.uuid4()),
            levels=[]
        )

        self.assertTrue("id" in export, export)

    def test_get_hierarchy(self):
        fi = self.hierarchy_mng.get_hierarchy(
                hierarchy_id=test_hierarchy["id"]
            )

        self.assertTrue("config" in fi, fi)
        self.assertTrue("id" in fi["config"], fi)

    def test_add_level(self):

        fi = self.hierarchy_mng.add_level(
                hierarchy_id=test_hierarchy["id"],
                level_name=str(uuid.uuid4())
            )

        self.assertTrue("groupName" in fi, fi)



# Copyright 2021 Element Analytics, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Contains helper methods
"""
import csv


def build_header(auth_token=None, org_id=None, others=None):
    """
    Creates request header

    :param auth_token: x-auth-toben to be included
    :type auth_token: str
    :param org_id: Org id where the request will be aimed
    :type org_id: int or str
    :param others: Other headers to be included.
    :type others: dict
    :return:
    """
    header = {}

    if others is not None and isinstance(others, dict):
        header.update(others)

    if auth_token is not None:
        header["x-auth-token"] = auth_token

    if org_id is not None:
        header["x-organization-id"] = str(org_id)

    return header


def tabulate_from_json(value):
    """
    Creates a table from json blob

    :param value: JSON
    :type value: dict
    :return:
    """
    tabs = {}

    for item in value:

        for header in item.keys():
            if header not in tabs:
                tabs[header] = []

            tabs[header].append(item[header])

    return tabs


def save_flow_download(name, data):
    """
    Stores the data into a temporary file

    :param name: File name
    :type name: str
    :param data: Contents
    :type data: str
    :return:
    """
    file_dir = f"{name}.csv"

    fil = open(file_dir, "wb+")

    fil.write(data)

    fil.close()

    return file_dir


def remove_special(out_file, data_array):
    """
    Removes special columns from the given file

    :param out_file: Directory where the new file is going to be written
    :type out_file: str
    :param data_array: list that contains the data to be removed
    :type data_array: list of json
    :return:
    """
    server_name_value = None

    with open(out_file, 'w+') as csv_r:

        csv_writer = csv.writer(csv_r, delimiter=',', quotechar='"', quoting=csv.QUOTE_ALL)

        first = True

        for row_json in data_array:

            if "dataArchiveServerName" in row_json:
                server_name_value = row_json["dataArchiveServerName"]
                del row_json["dataArchiveServerName"]

            if "metricsId" in row_json:
                del row_json["metricsId"]

            if first:
                first = False
                csv_writer.writerow(row_json.keys())

            csv_writer.writerow([valu for key, valu in row_json.items()])

    return server_name_value



# Copyright 2021 Element Analytics, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
This script contains some test cases for organization administration for the SDK
"""
import unittest
import os
import uuid

from tests import *


# from unify.orgadmin import OrgAdmin
# from unify.properties import Properties
# from unify.properties import ClusterSetting


class TestOrgAdmin(unittest.TestCase):

    def test_can_login(self):
        """
        Verify sdk can login
        :return:
        """
        org_admin = OrgAdmin(props=props, cluster=cluster_name)

        token = props.get_auth_token(cluster=cluster_name)

        org_admin.close_session()

        self.assertNotEqual(token, None, "Token is None")

    def test_can_create_org(self):
        """
        Verify sdk can create org
        :return:
        """
        org_admin = OrgAdmin(props=props, cluster=cluster_name)

        org_id = org_admin.create_organization(
            org_name="sdk-tests-{}".format(str(uuid.uuid4()))
        )

        org_admin.close_session()

        self.assertTrue(
            str(org_id).strip().isdigit(),
            "Org Id is created and its a number"
        )

    def test_get_org_info(self):
        """
        Verify sdk can get org info
        :return:
        """
        org_admin = OrgAdmin(props=props, cluster=cluster_name)

        org_id, _ = org_admin.get_org_info(
            org_id=test_org
        )

        org_admin.close_session()

        self.assertTrue("id" in org_id, org_id)
        self.assertEqual(str(org_id["id"]), str(test_org), org_id)


if __name__ == '__main__':
    unittest.main()



# Copyright 2021 Element Analytics, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Contains support methods for export and import workflows
"""
import json
import os.path
import uuid

from tempfile import mkstemp

from unify.properties import ClusterSetting
from unify.orgadmin import OrgAdmin
from unify.properties import Properties
from unify.templates import Templates
from unify.pipelines import Pipelines
from unify.sources import Sources
from unify.generalutils import json_to_csv
from unify.generalutils import csv_to_json
from unify.apiutils import remove_special
from unify.assetaccess import AssetAccess
from unify.graph import Graph
from unify.hierarchies import Hierarchy
from unify.helpers.SingleOrg import single_org
from unify.permissions import Permissions
from unify.helpers.Permissions import ArtifactType


class ApiManager:
    """
    Class to interact with import and export dataset methods
    """

    def __init__(self, cluster=None, props=Properties(ClusterSetting.KEY_RING), org_id=None):
        try:

            self.properties = props

            self.evergreen_enabled = self.properties.get_asset_sync(cluster=cluster)

            self.orgs = OrgAdmin(cluster=cluster, props=self.properties)

            self.templates = Templates(cluster=cluster, props=self.properties, org_id=org_id)

            self.pipelines = Pipelines(cluster=cluster, props=self.properties)

            self.sources = Sources(cluster=cluster, props=self.properties)

            self.graphs = Graph(cluster=cluster, props=self.properties)

            self.hierarchies = Hierarchy(cluster=cluster, props=self.properties)

            self.permissions = Permissions(cluster=cluster, props=self.properties)

            self.cluster = cluster

            self.sources.evergreen_enabled = self.evergreen_enabled

            self.pipelines.evergreen_enabled = self.evergreen_enabled

            self.org_id = org_id

        except Exception as error:
            raise error

    @single_org
    def delete_source(self, org_id=None, *, source_id):

        return self.sources.delete_source(
            org_id=org_id,
            source_id=source_id
        )

    def query_graph(self, org, graph, query, json_format):
        """
        Create a cypher query request

        :param org: Org where the graph to be queried is stored
        :param graph: Graph uuid to be queried
        :param query: String that contains cypher query
        :param json_format: return the results in a universal correctly json o json lines
        :return:
        """

        return self.graphs.query_graph(
            org_id=org,
            graph=graph,
            query=query,
            json_format=json_format
        )

    @single_org
    def graphs_list(self, org_id=None):
        """
        Return the list of graphs on a given org

        :param org_id: Org ID that wants to be queried
        :return: Json list with graph on the org
        """
        return self.graphs.get_graphs_list(org_id=org_id)

    @single_org
    def regular_pipeline_duplicate(self, org_id=None, *, pipeline_id, new_name):
        """
        Duplicate a given pipeline on the same org

        :param org_id: Org id where the pipeline is stored
        :param pipeline_id: Pipeline id that will be duplicated
        :param new_name: The new pipeline's name
        :return:
        """
        return self.pipelines.regular_duplicate(
            org_id=org_id,
            pipeline_id=pipeline_id,
            new_name=new_name
        )

    def import_source(self, content, orgid, name, type, encoding="UTF-8", chunks=10000):
        """
        Creates a new dataset with content. Option to chunk the amount of lines that the file will be partitioned

        :param content: CSV or TSV file content
        :type content: str
        :param orgid: Org where the source will be imported
        :type orgid: int or str
        :param name: Name to be given to the datasource
        :type name: str
        :param type: Type of the dataset. Accepts "external" or "generic"
        :type type: str
        :param encoding: File encoding
        :type encoding: str
        :param chunks: Number of chunks to split a large dataset,
        :type chunks: int
        :return:
        """
        if type == "external":

            file_dir, path = mkstemp(suffix=".csv")

            open(path, "wb").write(content.encode())

            new_source = self.sources.static_file_upload(
                name=name,
                content=path,
                org_id=orgid
            )

            os.close(file_dir)

            if "sourceMetadata" in new_source:
                del new_source["sourceMetadata"]

            return new_source

        elif type == "generic":

            return self.sources.upload_big_dataset(
                name=name,
                org_id=orgid,
                content=content,
                encoding=encoding,
                chunks=chunks
            )

        elif type == "split":
            return self.sources.divide_dataset_in(
                name=name,
                org_id=orgid,
                content=content,
                encoding=encoding,
                chunks=chunks
            )

        raise Exception("Unsupported type of source to be imported")

    @single_org
    def export_source(self, org_id=None, *, dataset_ids):
        """
        Retrieves the content of a dataset

        :param org_id: org id where the dataset is located
        :type org_id: int or str
        :param dataset_ids: dataset id to be retrieved
        :type dataset_ids: list of str
        :return:
        """
        return self.sources.create_export_dataset(
            org_id=org_id,
            dataset_ids=dataset_ids
        )

    def append_data(self, content, orgid, dataset_id):
        """
        Appends content to a dataset

        :param content: dataset comntent to be appended
        :type content: str
        :param orgid: org id where the target dataset lives
        :type orgid: int or str
        :param dataset_id: dataset id
        :type dataset_id: str
        :return:
        """
        return self.sources.append_dataset(
            org_id=orgid,
            data_set_id=dataset_id,
            content=content
        )

    @single_org
    def dataset_list(self, org_id=None):
        """
        Retrieves metadata for all datasets on a given org

        :param org_id: ORG id to be queried
        :type org_id: ORG id to be queried
        :return:
        """
        try:
            sources = self.sources.get_sources(org_id=org_id)

            for source in sources:
                if "schema" in source:
                    del source["schema"]

            return sources
        except Exception as error:
            raise error

    @single_org
    def pipeline_list(self, org_id=None):
        """
        Retrieves metadata for all pipelines on a given org

        :param org_id: ORG id to be queried
        :type org_id: ORG id to be queried
        :return:
        """
        try:
            return self.pipelines.get_pipelines_v2(org_id=org_id)

        except Exception as error:
            raise error

    def execute_query(self, query, orgid, format):
        """
        Executes the given sql query through Unify Access on a given org

        :param query: Query
        :type query: str
        :param orgid: Org id to query
        :type orgid: int or str
        :param format: Result format. Accepts "CSV".
        :type format: str
        :return:
        """

        access = AssetAccess(cluster=self.cluster, orgid=orgid, props=self.properties)

        result = access.execute_query(query=query)

        if format == "csv":
            return json_to_csv(result, io_format=False)

        return result

    def export_function(self, org_id, function):
        """
        Create a json structure with all the components needed to import a function

        :param org_id:
        :param function:
        :return:
        """
        return self.create_pipeline_export_data(
            org_id=org_id,
            pipeline_id=function,
            skip=[]
        )

    def assethub_access_tables(self, orgid):
        """
        Retrieves all Unify Access tables on the given org

        :param orgid: Org id to query the asset access tables
        :type orgid: int or str
        :return:
        """
        access = AssetAccess(cluster=self.cluster, orgid=orgid, props=self.properties)

        return access.get_all_tables()

    def import_sources(self, org_id, pipeline_id, content, handleduplicates="clone", update_pipeline=True):
        """
        Import the datasets, this method is used on the wf import pipelines

        :param org_id: Org id where the sources are going to be imported
        :type org_id: int or str
        :param pipeline_id: In case the sources that are imported and a pipeline used then
        :type pipeline_id: int or str
        :param content: Content file has all the data.
        :type content: str
        :param handleduplicates: In case there is a dataset with the same name. Accept "clone"
        :type handleduplicates: str, optional
        :param update_pipeline: Flag to update the pipeline
        :type update_pipeline: bool, optional
        :return:
        """

        try:
            data = json.loads(content)

            non_evergreen_conversion_type = {
                "JsonPiConfigFileSourceMetaData": "piconfig",
                "JsonFileSourceMetadata": "external",
                "JsonPiWebApiSourceMetadata": "web",
                "PI_Upload": "piconfig",
                "Upload": "external",
                "api_dataset": "api_dataset",
                "PiWeb": "web"
            }

            needed_keys = {"id", "component", "schema", "type", "file_content", "name"}

            already_created_by_name = {}

            msg = []

            for source_to_import in data:

                if needed_keys.issubset(source_to_import.keys()):

                    if source_to_import["type"] in non_evergreen_conversion_type:

                        which_type = non_evergreen_conversion_type[source_to_import["type"]]

                    else:
                        which_type = "api_dataset"

                    if which_type in ["api_dataset"]:
                        new_source = self.sources.create_api_data_set_with_content(
                            name=source_to_import["name"],
                            org_id=org_id,
                            content=json_to_csv(
                                source_to_import["file_content"],
                                io_format=False
                            )
                        )

                        self.sources.wait_for_dataset(org_id=org_id, dataset_id=new_source["data_set_id"])

                        source_to_import["new_source_id"] = new_source["data_set_id"]

                        msg.append("DataSource '{}' imported successfully".format(source_to_import["name"]))

                    if which_type in ["external"]:

                        file_dir, path = mkstemp(suffix=".csv")

                        open(path, "wb").write(
                            json_to_csv(
                                source_to_import["file_content"],
                                io_format=False
                            ).encode()
                        )

                        new_source = self.sources.static_file_upload(
                            name=source_to_import["name"],
                            content=path,
                            org_id=org_id
                        )

                        self.sources.wait_for_dataset(org_id=org_id, dataset_id=new_source["id"])

                        source_to_import["new_source_id"] = new_source["id"]

                        already_created_by_name[source_to_import["name"]] = new_source["id"]

                        msg.append("DataSource '{}' imported successfully".format(source_to_import["name"]))

                        os.close(file_dir)
                        try:
                            os.remove(path)
                        except OSError:
                            pass

                    if which_type in ["piconfig", "web"]:

                        file_dir, path = mkstemp(suffix=".csv")

                        server_name = remove_special(
                            out_file=path,
                            data_array=source_to_import["file_content"]
                        )

                        if server_name is None:
                            server_name = "PITAGCONFIGFILESERVER"

                        new_source = self.sources.pi_config_upload(
                            name=source_to_import["name"],
                            server_name=server_name,
                            file_path=path,
                            org_id=org_id
                        )

                        self.sources.wait_for_dataset(org_id=org_id, dataset_id=new_source["id"])

                        source_to_import["new_source_id"] = new_source["id"]

                        already_created_by_name[source_to_import["name"]] = new_source["id"]

                        msg.append("DataSource '{}' imported successfully".format(source_to_import["name"]))

                        os.close(file_dir)
                        try:
                            os.remove(path)
                        except OSError:
                            pass

                else:

                    raise Exception("Missing export data {} ".format(
                        needed_keys.difference(source_to_import.keys()))
                    )

            if update_pipeline:

                pipeline_info = self.pipelines.get_pipeline(
                    org_id=org_id,
                    pipeline_id=pipeline_id
                )

                for component in pipeline_info["pipeline"]["data"]["components"]:

                    if component["jsonClass"] == "JsonSourceRef":

                        for modified_source in data:

                            if modified_source["component"] == component["id"]:
                                component["sourceId"] = modified_source["new_source_id"]

                self.pipelines.update_pipeline_from_json(
                    org_id=org_id,
                    update_payload=pipeline_info["pipeline"]["data"],
                    pipeline_id=pipeline_id,
                    pipeline_name=pipeline_info["pipeline"]["data"]["name"]
                )

                return json.dumps(pipeline_info)

            return str(msg)

        except Exception as error:
            raise error

    def use_existing_datasets(self, org_id, pipeline_id, content):
        """
        Method to use the existing datasets in the pipeline to be imported

        :param org_id: Destination org where datasets live
        :type org_id: int or str
        :param pipeline_id: Pipeline which is being exported
        :type pipeline_id: int or str
        :param content: Contents
        :type content: str
        :return:
        """
        try:
            errors = set()
            pipeline_info = self.pipelines.get_pipeline(
                org_id=org_id,
                pipeline_id=pipeline_id
            )

            data = json.loads(content)

            sources = self.sources.get_sources(org_id=org_id)

            current_name_to_id = {}

            for source in sources:
                current_name_to_id[source["name"]] = source["id"]["id"]

            import_id_to_name = {}
            for source_to_import in data:
                import_id_to_name[source_to_import["id"]] = source_to_import["file_content"]

            for component in pipeline_info["pipeline"]["data"]["components"]:
                if component["jsonClass"] == "JsonSourceRef":

                    for modified_source in data:

                        if modified_source["component"] == component["id"]:
                            if import_id_to_name[component["sourceId"]] in current_name_to_id:

                                component["sourceId"] = current_name_to_id[
                                    import_id_to_name[component["sourceId"]]
                                ]

                            else:
                                errors.add(
                                    "No dataset with name {} was found".format(
                                        import_id_to_name[component["sourceId"]]
                                    )
                                )

                                component["sourceId"] = str(uuid.uuid4())

                self.pipelines.update_pipeline_from_json(
                    org_id=org_id,
                    update_payload=pipeline_info["pipeline"]["data"],
                    pipeline_id=pipeline_id,
                    pipeline_name=pipeline_info["pipeline"]["data"]["name"]
                )

        except Exception as err:
            raise err

        return list(errors)

    @single_org
    def create_export_source_file(self, org_id=None, *, pipeline_id, skip):
        """
        This methods creates the export file

        :param org_id: Org id to export the dataset
        :type org_id: str
        :param pipeline_id: Pipeline id
        :type pipeline_id: int or str
        :param skip: skip
        :type skip: list
        :return:
        """

        sources = self.sources.get_sources(org_id=org_id)

        source_id_to_data = {}

        result_data = []

        for source in sources:
            source_id_to_data[source["id"]["id"]] = source

        pipeline_info = self.pipelines.get_pipeline(org_id=org_id, pipeline_id=pipeline_id)

        for component in pipeline_info["pipeline"]["data"]["components"]:
            if component["jsonClass"] == "JsonSourceRef":
                source_data = {
                    "id": component["sourceId"],
                    "component": component["id"]
                }

                if component["sourceId"] in source_id_to_data:

                    if self.evergreen_enabled:

                        source_data["schema"] = source_id_to_data[source_data["id"]]["schema"]

                        source_data["name"] = source_id_to_data[source_data["id"]]["name"]

                        if "ean_source_type" in source_id_to_data[source_data["id"]]["labels"]["raw"]:
                            source_data["type"] = source_id_to_data[source_data["id"]]["labels"]["raw"][
                                "ean_source_type"]
                        else:
                            source_data["type"] = "api_dataset"

                    else:
                        source_data["name"] = source_id_to_data[source_data["id"]]["name"]
                        source_data["schema"] = source_id_to_data[source_data["id"]]["schema"]
                        source_data["type"] = source_id_to_data[source_data["id"]]["sourceMetadata"]["jsonClass"]

                    if skip:
                        source_data["file_content"] = source_id_to_data[source_data["id"]]["name"]
                    else:
                        source_original_data = self.sources.download_dataset_content(
                            org_id=org_id,
                            dataset_id=component["sourceId"]
                        )
                        source_data["file_content"] = csv_to_json(csv_data=source_original_data)

                    result_data.append(source_data)

        return json.dumps(result_data)

    @single_org
    def create_pipeline_export_data(self, org_id=None, *, pipeline_id, skip):
        """
        Create all the necessary data to export a pipeline

        :param org_id: Org id where the pipeline to be migrated is located
        :type org_id: int or str
        :param pipeline_id: Pipeline id to be exported
        :type pipeline_id: int or str
        :param skip: List of assets tp export, datasets and templates
        :type skip: list of str
        :return:
        """
        try:
            results = {}

            pipeline_info = self.pipelines.get_pipeline(org_id=org_id, pipeline_id=pipeline_id)

            templates_used = []
            aux_list = self.templates.list_asset_templates(org_id=org_id)

            mapped_templates = []

            results["map_attributes"] = {}

            results["functions"] = {}

            for component in pipeline_info["pipeline"]["data"]["components"]:

                if component["jsonClass"] == "JsonMapTemplates":

                    for mapped_value in component["values"]:
                        mapped_templates.append(mapped_value[1])

                if component["jsonClass"] == "JsonMapAttributes":
                    results["map_attributes"][component["id"]] = csv_to_json(
                        csv_data=self.pipelines.download_map_attributes(
                            org_id=org_id,
                            pipeline_id=pipeline_id,
                            component_id=component["id"]
                        )
                    )

                if component["jsonClass"] == "JsonFunctionApply":
                    func_id = "{}.{}.{}".format(pipeline_id, component["id"], component["pipelineId"])
                    results["functions"][func_id] = json.loads(
                        self.create_pipeline_export_data(
                            org_id=org_id,
                            pipeline_id=func_id,
                            skip=skip
                        )
                    )

            used_templates = []
            template_id_to_name = {}

            for temp in aux_list:

                if str(temp["id"]) in mapped_templates:
                    used_templates.append(temp["id"])
                    template_id_to_name[str(temp["id"])] = temp["name"]
                    if "updatedBy" in temp:
                        del temp["updatedBy"]
                    if "updated" in temp:
                        del temp["updated"]
                    templates_used.append(temp)

            results["pipeline"] = pipeline_info

            results["sources"] = json.loads(
                self.create_export_source_file(
                    org_id=org_id,
                    pipeline_id=pipeline_id,
                    skip="datasets" in skip
                )
            )

            if "templates" in skip:

                results["pipeline"], _ = self.change_template_ids_to_names(
                    pipeline_data=results["pipeline"],
                    templates_data=template_id_to_name
                )

                results["templates"] = []

                results["template_values"] = []

            else:

                results["templates"] = templates_used
                results["template_values"] = []
                if len(used_templates) > 0:
                    results["template_values"] = json.loads(
                        self.templates.download_given_templates(
                            org_id=org_id,
                            template_list=used_templates,
                            format="json")
                    )

            return json.dumps(results)

        except Exception as error:
            raise error

    def change_template_ids_to_names(self, pipeline_data, templates_data):
        """
        Changes the templates ids to its name

        :param pipeline_data: Pipeline json blob where templates are used
        :type pipeline_data: json
        :param templates_data: Templates data to be used
        :type templates_data: List of str
        :return:
        """

        errors = set()

        for component in pipeline_data["pipeline"]["data"]["components"]:

            if component["jsonClass"] == "JsonMapTemplates":

                if "values" in component:
                    new_val = []
                    for value in component["values"]:
                        row_vale = value[0]
                        template_id = value[1]
                        if template_id in templates_data:
                            new_val.append(
                                [
                                    row_vale,
                                    templates_data[template_id]
                                ]
                            )
                        else:
                            errors.add("{} not found".format(row_vale))

                    component["values"] = new_val

        return pipeline_data, list(errors)

    def mapped_rules_to_csv(self, rules):
        full_json = []

        if "distinctValues" in rules:
            distinct = rules["distinctValues"]
            if "cols" in distinct:
                cols = distinct["cols"]

                if "Count *" in cols:
                    cols.remove("Count *")

                if "data" in distinct:
                    # we have data :)
                    data = distinct["data"]
                    if "rows" in data:
                        raw_data = data["rows"]

                        for row in raw_data:
                            new_row = dict()

                            for col in cols:
                                new_row[col] = row[col]

                            full_json.append(new_row)

        return full_json

    def create_map_attributes_without_download(self, component, org_id, pipeline_id):
        if "displayInformation" in component:
            try:
                if component["displayInformation"] is not None:

                    if "columns" in component["displayInformation"]:

                        mapped_rules = self.pipelines.get_distinct_values_map_attributes(
                            org_id=org_id,
                            pipeline_id=pipeline_id,
                            flows=component["mappedFlow"]["id"],
                            columns=component["displayInformation"]["columns"]
                        )

                    else:
                        mapped_rules = self.pipelines.get_distinct_values_map_attributes(
                            org_id=org_id,
                            pipeline_id=pipeline_id,
                            flows=component["mappedFlow"]["id"],
                            columns=[]
                        )
                else:
                    mapped_rules = self.pipelines.get_distinct_values_map_attributes(
                        org_id=org_id,
                        pipeline_id=pipeline_id,
                        flows=component["mappedFlow"]["id"],
                        columns=[]
                    )

                return self.mapped_rules_to_csv(rules=mapped_rules)
            except Exception as err:
                pass

    def change_tempate_names_to_ids(self, pipeline_data, templates_data):
        """
        Change the templates names to its ids

        :param pipeline_data: Pipeline json blob where templates are used
        :type pipeline_data: int or str
        :param templates_data: Templates data to be used
        :type templates_data: List of str
        :return:
        """

        errors = set()

        for component in pipeline_data["pipeline"]["pipeline"]["data"]["components"]:

            if component["jsonClass"] == "JsonMapTemplates":

                if "values" in component:
                    new_val = []
                    for value in component["values"]:
                        row_vale = value[0]
                        template_name = value[1]
                        if template_name in templates_data:
                            new_val.append(
                                [
                                    row_vale,
                                    str(templates_data[template_name])
                                ]
                            )
                        else:
                            errors.add("{} not found".format(template_name))

                    component["values"] = new_val

        return pipeline_data, list(errors)

    @single_org
    def proceses_importing_pipeline_file(self, org_id=None, *, content, skip, handleduplicates="clone", function=False,
                                         loaded=False):
        """
        Process pipeline importing.

        :param org_id: Target org id where the pipelien will be imported
        :type org_id: int or str
        :param content: Pipeline import content
        :type content: str
        :param skip: variable to skip datasets or templates
        :type skip: list
        :param handleduplicates: Boolean variable to decide what to do with duplicate pipeline
        :type handleduplicates: bool, optional
        :return:
        """
        try:
            errors = []
            if not loaded:
                data = json.loads(content)
            else:
                data = content

            if handleduplicates == "clone":

                temp_name = data["pipeline"]["pipeline"]["data"]["name"]

                if self.pipelines.pipeline_exists(org_id=org_id, pipeline_name=temp_name):
                    data["pipeline"]["pipeline"]["data"]["name"] = "{}_{}".format(
                        temp_name,
                        str(uuid.uuid4())[:5]
                    )

            if "pipeline" not in data:
                raise Exception("Pipeline data was not found")

            pipeline_info = data["pipeline"]

            migrated_funcs = {}
            if "functions" in data:
                for func_id, func_content in data["functions"].items():
                    exists = self.pipelines.verify_if_pipeline_exists_and_get_id(
                        org_id=org_id,
                        pipeline_name=func_content["pipeline"]["pipeline"]["data"]["name"]
                    )
                    keys = func_id.split(".")

                    if exists["pipeline_id"] is not None:
                        migrated_funcs[keys[-1]] = exists["pipeline_id"]

                    if func_id not in migrated_funcs and exists["pipeline_id"] is None:
                        resp = json.loads(
                            self.proceses_importing_pipeline_file(
                                org_id=org_id,
                                content=func_content,
                                skip=skip,
                                handleduplicates=handleduplicates,
                                function=True,
                                loaded=True
                            )
                        )

                        migrated_funcs[keys[-1]] = resp["pipeline_id"]

            if function:
                pipeline_info["pipeline"]["data"]["pipelineType"] = "function"
            else:
                if "pipelineType" in pipeline_info["pipeline"]["data"]:
                    del pipeline_info["pipeline"]["data"]["pipelineType"]

            if "templates" not in data:
                raise Exception("Template data was not found")

            if "template_values" not in data:
                raise Exception("Template definitions was not found")

            if len(data["template_values"]) > 0:
                try:

                    self.templates.upload_string_content_file(
                        content=json_to_csv(
                            data_array=data["template_values"],
                            io_format=False
                        ),
                        org_id=org_id
                    )

                except Exception as error:
                    errors.append("Exporting many templates can cause issues!")

            template_info = data["templates"]

            templates = self.templates.list_asset_templates(org_id=org_id)

            current_id_name_map = {}
            prev_id_name_map = {}

            for template in template_info:
                prev_id_name_map[str(template["id"])] = template["name"]

            for template in templates:
                current_id_name_map[template["name"]] = template["id"]

            if "templates" in skip:
                data, err = self.change_tempate_names_to_ids(
                    pipeline_data=data,
                    templates_data=current_id_name_map
                )

                errors.extend(err)

            attributes_name_to_id = {}

            data["map_recipes"] = {}

            for used_template in data["templates"]:

                current_temp_id = current_id_name_map[used_template["name"]]

                attrs = self.templates.get_attributes(
                    org_id=org_id,
                    template_id=current_temp_id
                )

                attributes_name_to_id[current_temp_id] = {}

                for item in attrs["items"]:
                    attributes_name_to_id[current_temp_id][item["name"]] = item["id"]

            for component in pipeline_info["pipeline"]["data"]["components"]:

                if component["jsonClass"] == "JsonMapTemplates":
                    if "templates" not in skip:
                        for mapped_value in component["values"]:
                            if mapped_value[1] in prev_id_name_map:
                                inside = prev_id_name_map[mapped_value[1]]
                                if inside in current_id_name_map:
                                    mapped_value[1] = str(
                                        current_id_name_map[inside]
                                    )
                                else:
                                    errors.append(
                                        ("Template {} was deleted and "
                                         "is still mapped".format(mapped_value[1]))
                                    )
                            else:
                                errors.append(
                                    ("Template {} was deleted and "
                                     "is still mapped".format(mapped_value[1]))
                                )

                if component["jsonClass"] == "JsonMapAttributes":
                    if component["id"] in data["map_attributes"]:
                        data["map_recipes"][component["id"]] = data["map_attributes"][component["id"]]

                if component["jsonClass"] == "JsonFunctionApply":
                    component["pipelineId"] = migrated_funcs[str(component["pipelineId"])]

                if component["jsonClass"] == "JsonGraphsV2Sink":
                    component["orgId"] = org_id
                    component["graphId"] = str(uuid.uuid4())

                if component["jsonClass"] == "JsonDatasetsSink":
                    component["orgId"] = org_id
                    component["datasetId"] = str(uuid.uuid4())

            pipeline = self.pipelines.create_pipeline(
                org_id=org_id,
                name=data["pipeline"]["pipeline"]["data"]["name"],
                function=function
            )

            created_pipeline_id = pipeline['pipeline']['id']

            self.pipelines.wait_for_pipeline(
                org_id=org_id,
                pipeline_id=created_pipeline_id
            )

            self.pipelines.update_pipeline_from_json(
                org_id=org_id,
                update_payload=data["pipeline"]["pipeline"]["data"],
                pipeline_id=created_pipeline_id,
                pipeline_name=data["pipeline"]["pipeline"]["data"]["name"]
            )

            for component_id, recipe in data["map_recipes"].items():
                try:
                    self.pipelines.upload_map_attributes_from_json(
                        org_id=org_id,
                        component_id=component_id,
                        pipeline_id=created_pipeline_id,
                        json_data=recipe
                    )
                except Exception as err:
                    errors.append(repr(err))

            if "sources" in data:
                if "datasets" not in skip:
                    self.import_sources(
                        org_id=org_id,
                        pipeline_id=created_pipeline_id,
                        content=json.dumps(data["sources"]),
                        handleduplicates=handleduplicates,
                    )

                else:

                    err = self.use_existing_datasets(
                        org_id=org_id,
                        pipeline_id=created_pipeline_id,
                        content=json.dumps(data["sources"]),
                    )
                    errors.append(err)

            results = {
                "pipeline_id": created_pipeline_id,
                "org_id": org_id,
                "url": "{}#/org/{}/pipelines/{}".format(
                    self.properties.get_remote(self.cluster),
                    org_id,
                    created_pipeline_id
                ),
                "warnings": errors
            }

            return json.dumps(results)
        except Exception as error:
            raise error

    @single_org
    def get_all_hierarchies_display(self, org_id=None):
        """
        Method to retrieve all the organization on a given org

        :param org_id: Organization id where the org is stored
        :return:
        """
        columns = [
            "id", "name"
        ]

        raw_data = self.get_all_hierarchies(org_id=org_id)

        for data in raw_data:

            for key in list(data.keys()):

                if key not in columns:
                    del data[key]

        return raw_data

    @single_org
    def get_single_hierarchy(self, org_id=None, *, hierarchy):
        """
        Method to retrieve all the data form a single hierarchy and remove unnecessary data to be displayed on the console

        :param org_id: Organization id where the org is stored
        :param hierarchy: Hierarchy id to be retrieved
        :return:
        """
        try:
            columns = [
                "id", "name", "levels"
            ]

            raw_data = self.hierarchies.get_hierarchy(
                org_id=org_id,
                hierarchy_id=hierarchy
            )["config"]

            for key in list(raw_data.keys()):

                if key not in columns:
                    del raw_data[key]

            return raw_data

        except Exception as error:
            raise error

    @single_org
    def get_all_hierarchies(self, org_id=None):
        """
        Method to retrieve all the organization on a given org

        :param org_id: Organization id where the org is stored
        :return:
        """
        try:
            return self.hierarchies.get_all_hierarchies(
                org_id=org_id
            )
        except Exception as error:
            raise error

    @single_org
    def create_hierarchy(self, org_id=None, *, name, levels=None, attribute_names=None, private=False):
        """
        Method to create a new hierarchy on the given org

        :param org_id: Organization id where the org would be saved
        :param name: String name of the new hierarchy
        :param levels: Array containing the level names
        :param attribute_names:
        :param private: Boolean value to specify if the hierarchy is private
        :return:
        """
        try:
            return self.hierarchies.create_hierarchy(
                org_id=org_id,
                name=name,
                levels=levels if levels is not None else [],
                attribute_names=attribute_names,
                private=private
            )
        except Exception as error:
            raise error

    @single_org
    def export_hierarchy(self, org_id=None, *, hierarchy):
        """
        Method to extract the necessary information to export a hierarchy

        :param org_id: organization id where the hierarchy currently is stored
        :param hierarchy: hierarchy id to be exported
        :return:
        """
        try:
            data = self.get_single_hierarchy(
                org_id=org_id,
                hierarchy=hierarchy
            )

            full_data = {
                "name": data["name"],
                "levels": dict()
            }

            for level in data["levels"]:
                full_data["levels"][level["index"]] = level['groupName']

            return json.dumps(full_data)
        except Exception as error:
            raise error

    @single_org
    def import_hierarchy(self, org_id=None, *, content):
        """
        Method to import a hierarchy, this should be used with the output of the export hierarchy

        :param org_id: organization id where the new hierarchy will be saved
        :param content: json object containing hierarchy data
        :return:
        """
        try:
            data = json.loads(content)
            if "name" in data:

                new_h = self.create_hierarchy(
                    org_id=org_id,
                    name=data["name"]
                )

                hierarchy_id = new_h["id"]

                if "levels" in data:
                    levels = data["levels"]

                    for i in range(0, len(levels.keys())):
                        self.hierarchies.add_level(
                            org_id=org_id,
                            hierarchy_id=hierarchy_id,
                            level_name=levels[str(i)]
                        )

                return new_h

        except Exception as error:
            raise error

    @single_org
    def get_rules_list(self, org_id=None):
        """

        Retrieves the org rules
        :param org_id: organization id to be queried
        :return:
        """

        resp = self.permissions.get_rules(org_id=org_id)

        return resp["data"]["permissions"]["getRules"]

    @single_org
    def get_selectors_list(self, org_id=None):
        """

        Retrieves selectors
        :param org_id: organization id to be queried
        :return:
        """

        resp = self.permissions.get_selector_definitions(org_id=org_id)

        return resp["data"]["permissions"]["getSelectorDefinitions"]

    @single_org
    def get_permissions_org_config(self, org_id=None):
        """

        Retrieves org permissions config
        :param org_id: organization id to be queried
        :return:
        """

        resp = self.permissions.get_org_config(org_id=org_id)

        return resp["data"]["permissions"]["getOrgConfig"]

    @single_org
    def get_pipeline_rules(self, org_id=None, *, pipeline_id):
        resp = self.permissions.get_artifact_rules(
            org_id=org_id, artifact_id=pipeline_id, artifact_type=ArtifactType.pipeline
        )

        return resp

    @single_org
    def get_dataset_rules(self, org_id=None, *, dataset_id):
        resp = self.permissions.get_artifact_rules(
            org_id=org_id, artifact_id=dataset_id, artifact_type=ArtifactType.dataset
        )

        return resp["data"]["permissions"]["getArtifactRules"]

    @single_org
    def check_permission(self, org_id=None, *, artifact_type, artifact_id, user_id, verb):
        resp = self.permissions.check_permission(
            org_id=org_id,
            artifact_type=artifact_type,
            artifact_id=artifact_id,
            user_id=user_id,
            verb=verb
        )

        return resp["data"]["permissions"]["check"]["allowed"]

    @single_org
    def get_selector_definition(self, org_id=None, *, selector_id):
        resp = self.permissions.get_selector_definition(
            org_id=org_id,
            selector_id=selector_id
        )
        return resp["data"]["permissions"]['getSelectorDefinition']

    @single_org
    def add_rule(self, org_id=None, *, domain, verb, effect, userSelector, resourceSelector):
        resp = self.permissions.add_rule(
            org_id=org_id,
            domain=domain,
            verb=verb,
            effect=effect,
            userSelector=userSelector,
            resourceSelector=resourceSelector
        )
        return resp["data"]["permissions"]

    @single_org
    def delete_rule(self, org_id=None, *, rule_id):

        resp = self.permissions.delete_rule(
            org_id=org_id,
            rule_id=rule_id
        )

        return resp["data"]["permissions"]



import json
from typing import Dict, List, Union

from gql_query_builder import GqlQuery


class GraphQLBuilder:

    @staticmethod
    def build_input_here(self, input: Dict[str, Union[str, int]], initial_str: str):
        inputs: List[str] = []

        final_str = initial_str

        if input != {}:
            key = list(input.keys())[0]
            nested_keys = list()

            while isinstance(input[key], dict):
                nested_keys.append(key)
                input = input[key]
                key = list(input.keys())[0]

            for key, value in input.items():
                if nested_keys:
                    if type(value) is str:
                        inputs.append(f'{key}: "{value}"')  # Nested input won't have double quotes
                    else:
                        inputs.append(f'{key}: {value}')  # Nested input won't have double quotes

                else:
                    inputs.append(f'{key}: {value}')

            final_str += '('

            for key in nested_keys:
                final_str = final_str + key + ': {'

            final_str = final_str + ", ".join(inputs)

            for _ in nested_keys:
                final_str += '}'

            final_str += ')'

        return final_str

    def build_generic_query(
            self,
            query_name=None,
            query_input: dict = None,
            query_alias=None,
            fields: list = None,
            field_alias: str = None,
            operation_type=None,
            operation_name=None,
            operation_queries: list = None,
            operation_input: dict = None,
    ):
        query = GqlQuery()

        operation_params = dict()
        build_oper = False
        if operation_name:
            operation_params["name"] = operation_name
            build_oper = True
        if operation_type:
            operation_params["query_type"] = operation_type
            build_oper = True
        if operation_input:
            operation_params["input"] = operation_input
            build_oper = True
        if operation_queries:
            operation_params["queries"] = operation_queries
            build_oper = True

        if build_oper:
            query = query.operation(**operation_params)

        query_params = dict()
        build_query = False
        if query_alias:
            query_params["alias"] = query_alias
            build_query = True

        if query_name:
            query_params["name"] = query_name
            build_query = True

        if query_input:
            query_params["input"] = query_input
            build_query = True

        if build_query:
            query = query.query(**query_params)

        if fields:
            query.fields(fields=fields)

        return query.generate()

    def build_fragment(self, fields, fragment_name, interface_name):

        return GqlQuery().fields(fields).fragment(fragment_name, interface_name).generate()

    def build_simple_query(self, query_name, query_input=None, query_alias=None, fields: list = None):

        query = GqlQuery()

        query_params = dict()

        if query_alias:
            query_params["alias"] = query_alias

        if query_name:
            query_params["name"] = query_name

        if query_input:
            query_params["input"] = query_input

        if fields:
            query.fields(fields=fields)

        query = query.query(**query_params)

        return query.generate()

    def build_last_updated_fragment(self):
        return self.build_fragment(
            fragment_name="LastUpdatedParts",
            interface_name="Artifact",
            fields=[
                self.build_simple_query(
                    query_name="lastUpdated",
                    fields=[
                        self.build_simple_query(
                            query_name="auditee",
                            fields=[
                                "id",
                                self.build_simple_query(
                                    query_name="details",
                                    fields=[
                                        "name",
                                        "__typename"
                                    ]
                                ),
                                "__typename"
                            ]
                        ),
                        "timestamp",
                        "__typename"
                    ]
                ),
                "__typename"
            ]
        )

    def build_common_parts_fragment(self, last_updated=True):
        fields = [
            "uuid",
            "name",
            "orgId",
            self.build_simple_query(
                query_name="id",
                fields=[
                    "id",
                    "type",
                    "__typename"
                ]
            ),
            "facets",
            "description",
            "__typename",

        ]

        if last_updated:
            fields.append("...LastUpdatedParts")

        return self.build_fragment(
            fields=fields,
            fragment_name="CommonParts",
            interface_name="Artifact"
        )

    def mutation_data_build(self, new_name=None, facets: list = None, description: str = None):

        values = dict()

        if facets:
            values["ean_facets"] = facets

        if description:
            values["ean_description"] = description

        return self.mutation_data_build_by_dict(
            new_name=new_name,
            values=values
        )

    def mutation_data_build_by_dict(self, new_name, values: dict):
        data = dict()

        if new_name:
            data["name"] = new_name

        add = []

        for key, value in values.items():
            add.append(
                {
                    "key": key,
                    "value": json.dumps(value)
                }
            )

        if len(add) > 0:
            data["added"] = add

        return data

    def build_artifact_mutation(self, artifact_id, artifact_type="dataset", data=None):
        """
        This builds the mutation graphql query
        :param artifact_id: pipeline or dataset id to be modified
        :return:
        """

        common_parts = self.build_common_parts_fragment()

        LastUpdatedParts = self.build_last_updated_fragment()

        artifacts = self.build_simple_query(
            query_name="updateArtifact",
            query_input={"id": "$id", "data": "$data"},
            fields=[
                "seqNum",
                self.build_simple_query(
                    query_name="artifact",
                    fields=[
                        "...CommonParts",
                        "__typename"
                    ]
                ),
                "__typename"
            ]
        )

        final = self.build_generic_query(
            operation_name="updateArtifact",
            operation_type="mutation",
            operation_input=
            {
                "$id": "ArtifactIdInput!",
                "$data": "ArtifactInput!"
            },
            operation_queries=[artifacts]
        )

        final_query = "".join(
            [final, common_parts, LastUpdatedParts]
        )

        results = {
            "operationName": "updateArtifact",
            "variables": {
                "id": {
                    "id": artifact_id,
                    "type": artifact_type
                },
                "data": data
            },
            "query": final_query,
        }

        return json.dumps(results)

    def build_wait_for_query(self, artifact_id, artifact_type="pipeline"):

        artifacts = self.build_simple_query(
            query_name="waitFor",
            query_input={"id": "$artifactId", "seqNum": "$seqNum", "timeoutSeconds": "$timeoutSeconds"},
            fields=[
                "...CommonParts"
            ]
        )

        common_parts = self.build_common_parts_fragment(last_updated=False)

        final = self.build_generic_query(
            operation_name="waitForArtifact",
            operation_type="query",
            operation_input={"$artifactId": "ArtifactIdInput!", "$seqNum": "Long!", "$timeoutSeconds": "Int!"},
            operation_queries=[artifacts]
        )

        final_query = ''.join(
            [final, common_parts]
        )

        results = {
            "operationName": 'waitForArtifact',
            "query": final_query,
            "variables": {
                "artifactId": {
                    "id": artifact_id if isinstance(artifact_id, str) else str(artifact_id),
                    "type": artifact_type
                },
                "seqNum": 0,
                "timeoutSeconds": 1800
            }
        }

        return json.dumps(results).replace("'", "")

    def mutation_query(self, artifact_id, artifact_type, new_name=None, facets: list = None, description: str = None):

        data = self.mutation_data_build(
            new_name=new_name,
            facets=facets,
            description=description
        )

        query = self.build_artifact_mutation(
            artifact_id=artifact_id,
            data=data,
            artifact_type=artifact_type
        )

        return query

    def build_artifacts_query(self, pipeline):
        return self.build_simple_query(
            query_name="artifacts",
            query_input={"pagination": "$pagination"},
            fields=[
                "...CommonParts",
                "... on",
                self.build_simple_query(
                    query_name="Pipeline" if pipeline else "Dataset",
                    fields=[
                        "...PipelineParts" if pipeline else "...DatasetParts",
                        "__typename"
                    ]
                ),
                "__typename"
            ]
        )


GqlQuery.build_input = GraphQLBuilder.build_input_here



# Copyright 2021 Element Analytics, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
This file contains static SQL queries that can be used in asset access
"""
ALL_TABLES = (
    "SELECT relname AS DataBase "
    "FROM pg_catalog.pg_class "
    "WHERE relnamespace={} ORDER BY relname DESC"
)

OID_QUERY = (
    "SELECT oid FROM pg_catalog.pg_namespace "
    "WHERE nspname='org_{}'"
)



# Copyright 2021 Element Analytics, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
import json

from tests import *
from unify.assetaccess import AssetAccess


class AssetAccessTest(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.access = AssetAccess(
            cluster=cluster_name,
            props=props,
            orgid=test_org
        )

    @unittest.skip("Asset Access is currently un-accessible")
    def test_get_all_tables(self):
        tables = self.access.get_all_tables()

        self.assertTrue(len(tables) > 0, tables)

    @unittest.skip("Asset Access is currently un-accessible")
    def test_execute_query(self):
        tables = self.access.get_all_tables()

        self.assertTrue(len(tables) > 0, tables)

        table = tables[0]["database"]

        results = self.access.execute_query(
            query='select * from {}'.format(table)
        )
        self.assertTrue(len(results) > 1, results)



# Copyright 2021 Element Analytics, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Contains methods to interact with graphs api
"""
import json
import io

import json_lines

from unify.properties import Properties
from unify.properties import ClusterSetting
from unify.apirequestsmng import ApiRequestManager
from unify.helpers.SingleOrg import single_org


class Graph(ApiRequestManager):
    """
    Class to interact with pipeline endpoints
    """

    def __init__(self, cluster=None, props=Properties(ClusterSetting.KEY_RING), org_id=None):

        """
        Class constructor

        :param cluster: Cluster name to be used
        :type cluster: str
        :param props: Properties instantiated object
        :type props: class:`unify.properties.Properties`
        """
        super().__init__(cluster=cluster, props=props, org_id=org_id)

        remote = self.props.get_remote(self.cluster)

        self.graph_url = remote + 'graph/v1'

        self.graph_query_url = self.graph_url + "/{}/query"

        self.export_query_url = self.graph_query_url + "/export"

        self.graph_merge_url = self.graph_url + "/merge"

        #https://aws-stage-qa-1.ean.io/graph/v1/merge?name=Try+building+a+new+graph

    @single_org
    def get_graphs_list(self, org_id=None):
        """
        Retrieves the list of graphs on the given org

        :param org_id: organization Id to be queried
        :return: Array containing list of graphs data
        """

        header = self.build_header(
            org_id=org_id,
            aut_token=self.props.get_auth_token(cluster=self.cluster),
            others=self.content_type_header
        )

        result = self.session.get(self.graph_url, headers=header)

        if result.status_code == 200:
            return json.loads(result.content)

        raise Exception(repr(result.content))

    @single_org
    def query_graph(self, org_id=None, *, graph, query, json_format=False):
        """
        Runs a cypher query on the given graph

        :param org_id: Organization id where the graph is stored
        :param graph: Graph id to be queried
        :param query: Cypher query
        :param json_format: return the results in a universal correctly json o json lines
        :return:
        """
        header = self.build_header(
            org_id=org_id,
            aut_token=self.props.get_auth_token(cluster=self.cluster),
            others=self.content_octet
        )

        final_url = self.graph_query_url.format(graph)

        result = self.session.post(final_url, headers=header, data=query)

        if result.status_code in [200, 201, 202]:
            iobytes = io.BytesIO(result.content)
            full_graph = []
            if json_format:
                for item in json_lines.reader(iobytes):
                    full_graph.append(item)

                return full_graph

            return result.content

        raise Exception(repr(result.content))

    @single_org
    def export_query(self, org_id=None, *, graph, query, nodes_dataset_name, edges_dataset_name):
        """
        Export a given cypher query into 2 datasets

        :param org_id: Organization id where the graph is stored
        :param graph: Graph id to be queried
        :param query: Cypher query
        :return:
        """

        header = self.build_header(
            org_id=org_id,
            aut_token=self.props.get_auth_token(cluster=self.cluster),
            others=self.content_octet
        )

        final_url = self.export_query_url.format(graph)

        data = (["nodesDatasetName", nodes_dataset_name], ["edgesDatasetName", edges_dataset_name])

        result = self.session.post(final_url, headers=header, data=query, params=data)

        if result.status_code in [200, 201, 202]:
            return result.content

        raise Exception(json.loads(result.content))

    @single_org
    def create_empty_graph(self, org_id=None, *, graph_name):
        """
        Create an empty graph with the given name

        :param org_id: Organization id where the graph would be stored
        :param graph_name: Empty graph name
        :return:
        """

        header = self.build_header(
            org_id=org_id,
            aut_token=self.props.get_auth_token(cluster=self.cluster),
            others=self.content_type_header
        )

        data = {"name": graph_name}

        result = self.session.post(self.graph_merge_url, headers=header, params=data)

        if result.status_code in [200, 201, 202]:
            return json.loads(result.content)

        raise Exception(json.loads(result.content))



# Copyright 2021 Element Analytics, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Contains methods to interact with organization api
"""

import json
import uuid

from unify.apirequestsmng import ApiRequestManager
from unify.properties import Properties
from unify.properties import ClusterSetting
from unify.helpers.SingleOrg import single_org


class OrgAdmin(ApiRequestManager):
    """
    Class to interact with organization endpoints
    """

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        pass

    def __init__(self, cluster=None, props=Properties(ClusterSetting.KEY_RING), org_id=None):
        """
        Class constructor

        :param cluster: Cluster name to be used
        :type cluster: name
        :param props: Properties instantiated object
        :type props: class:`unify.properties.Properties`
        """

        super().__init__(cluster=cluster, props=props, org_id=org_id)

        try:

            remote = self.props.get_remote(self.cluster)

            self.orgs_api_url = remote + 'api/management/v1/orgs'
            self.org_api_url = remote + 'api/management/v1/orgs/{}'
            self.org_info_url = remote + 'api/v1/orgs/{}'
            self.who_ami_url = remote + 'api/whoami'
            self.org_sensor_analytics = remote + "api/v1/orgs/{}/sensor_diagnostics"
            self.change_group_url = remote + "api/authz/v1/groups/{}/users"
            self.all_users_url = remote + "api/v1/users"
            self.all_users_on_org = remote + "api/v1/orgs/{}/users"
            self.all_groups_url = remote + "api/authz/v1/groups"

        except Exception as error:
            raise error

    @single_org
    def get_org_info(self, org_id=None):
        """
        Retrieves the org info

        :param org_id: Org id to be queried
        :type org_id: id or str
        :return: List of information on the organization
        """

        self.verify_properties(cluster_name=self.cluster)

        headers = self.build_header(
            org_id=org_id,
            aut_token=self.props.get_auth_token(cluster=self.cluster),
            others=self.content_type_header
        )

        jobs = self.session.get(
            self.org_info_url.format(org_id),
            headers=headers
        )

        return json.loads(jobs.content), jobs.status_code

    @single_org
    def submit_sensor_diagnostics(self, org_id=None, *, content):
        """
        Create a request to the sensor diagnostics endpoint

        :param org_id: Organization where the sensor diagnostics will be submitted
        :param content: sensot diagnostics data
        :return: Results of the sensor diagnostics request
        """
        try:

            header = self.build_header(
                aut_token=self.props.get_auth_token(cluster=self.cluster),
                others=self.content_type_header
            )

            submit_report = self.session.put(
                self.org_sensor_analytics.format(org_id),
                headers=header,
                data=content
            )

            if submit_report.status_code == 200:
                return json.loads(submit_report.content)

            raise Exception(submit_report.content)

        except Exception as error:

            raise error

    def create_organization(self, org_name):
        """
        Creates an organization with the given name

        :param org_name: Name or organization to be created
        :type org_name: str
        :return: Server response or exception if org name already exists
        """

        try:

            if org_name is None:
                org_name = str(uuid.uuid4())

            header = self.build_header(
                aut_token=self.props.get_auth_token(cluster=self.cluster),
                others=self.content_type_header
            )

            payload = {
                "name": str(org_name),
                "piTagExportLimit": self.pi_tag_export_limit,
            }

            org_create_post = self.session.post(
                self.orgs_api_url,
                headers=header,
                data=json.dumps(payload)
            )

            if org_create_post.status_code == 200:
                return json.loads(org_create_post.content)["id"]

            raise Exception(org_create_post.content)

        except Exception as error:

            raise error

    @single_org
    def delete_organization(self, org_id=None):
        """
        Deletes the org that matched the given org id

        :param org_id: Org id to be deleted
        :type org_id: int or str
        :return: Delete successful message
        """
        try:

            delete_endpoint = self.props.get_remote(
                cluster=self.cluster
            ) + 'api/management/v1/orgs/' + str(org_id)

            header = self.build_header(
                aut_token=self.props.get_auth_token(cluster=self.cluster),
                others=self.content_type_header
            )

            delete_request = self.session.delete(
                delete_endpoint,
                headers=header
            )

            if delete_request.status_code == 200:
                return json.loads(delete_request.content)

            raise Exception(json.loads(delete_request.content))

        except Exception as error:
            raise error

    @single_org
    def invite_user(self, org_id=None, *, email, name, role):
        """
        Adds an user to the given org

        :param org_id: Org id where the user is going to be added
        :type org_id: int or str
        :param email: User's email
        :type email: str
        :param name: User's Name
        :type name: str
        :param role: User role. Accepts "Admin" or "Contributor"
        :type role: str
        :return: Invite user status message
        """
        try:

            invite_user_url = self.org_info_url.format(org_id) + '/users'

            header = self.build_header(
                org_id=org_id,
                aut_token=self.props.get_auth_token(cluster=self.cluster),
                others=self.content_type_header
            )

            payload = {"fullName": name, "email": email, "roleNames": [role]}

            invite_user_post = self.session.post(
                invite_user_url,
                headers=header,
                data=json.dumps(payload)
            )

            if invite_user_post.status_code == 201:
                return json.loads(invite_user_post.content)

            raise Exception(json.loads(invite_user_post.content))

        except Exception as error:

            raise error

    @single_org
    def invite_machine_user(self, org_id=None, *, id, password, fullname, role):
        """
        Invites a machine user

        :param org_id: Org identification where the user is going to be added
        :type org_id: int or str
        :param id: User's ID
        :type id: int or str
        :param password: User's Password
        :type password: str
        :param role: User role. Accepts "Admin" or "Contributor"
        :type role: str
        :return: Invite machine user status message
        """
        try:

            invite_machine_user_url = self.org_info_url.format(org_id) + '/machine_users'

            header = self.build_header(
                org_id=org_id,
                aut_token=self.props.get_auth_token(cluster=self.cluster),
                others=self.content_type_header
            )

            payload = {
                "identifier": id,
                "password": password,
                "roleNames": [role],
                "fullName": fullname
            }

            invite_user_post = self.session.post(
                invite_machine_user_url,
                headers=header,
                data=json.dumps(payload)
            )

            if invite_user_post.status_code == 200:
                return json.loads(invite_user_post.content)

            raise Exception(json.loads(invite_user_post.content))

        except Exception as error:
            raise error

    def get_org_list(self):
        """
        Retrieves the org list of the current cluster, only the ones whos users has logged in

        :return: List of organizations
        """
        try:

            header = self.build_header(
                aut_token=self.props.get_auth_token(cluster=self.cluster)
            )

            request = self.session.get(
                self.who_ami_url,
                headers=header
            )

            if request.status_code == 200:
                return json.loads(request.content)["organizations"]

            raise Exception(request.content)

        except Exception as error:
            raise error

    @single_org
    def update_org_metadata(self, org_id, metadata, merge=True):
        """
        Update metadata for an an existing organization.

        * Can either overwrite whole metadata object, or patch into an existing
        (or empty, it's an upsert) organization.
        * If the types mismatch, and you specify merge=True this method throw
        an error. `merge=False` will overwrite the existing contents. Type
        mismatch is either assigning a scalar to an object/list, or vice versa.
        * If the metadata field in the object is not present, it will be treated
        like an empty object.

        Examples:

        If an existing org looks something like this:
        ```
        {
            // Stuff here
            metadata: {
                "baz": {
                    "biz": "buz"
                }
            }
        }
        ```

        This `o.update_org_metadata(81, '{"baz": {"bam": "batman"}}')` will
        yield this org:
        ```
        {
            // Stuff here
            metadata: {
                "baz": {
                    "biz": "buz",
                    "bam": "batman"
                }
            }
        }
        ```

        When merge is set to false, the results are different.

        This `o.update_org_metadata(81, '{"baz": {"bam": "batman"}}', merge=False)`
        will yield this org:
        ```
        {
            // Stuff here
            metadata: {
                "baz": {
                    "bam": "batman"
                }
            }
        }
        ```

        :param org: Operates on the specified org id
        :param metadata: Contents (json string) of the metadata field on an
                         org. Does not include the key, "metadata"
        :param merge: Boolean indicating whether to merge into the existing
                      metadata object or replace the contents.
        :return:
        """

        header = self.build_header(
            aut_token=self.props.get_auth_token(cluster=self.cluster),
            others=self.content_type_header
        )

        organization_url = self.org_api_url.format(org_id)

        metadata = json.loads(metadata)

        try:

            org_info, status_code = self.get_org_info(org_id)

            if status_code not in [200, 201]:
                raise Exception(org_info)

            old_metadata = org_info.get("metadata", {})

            if merge:
                def meta_merge(new_m, old_m, path_acc):
                    def cmp(comp_type):
                        return isinstance(new_m, comp_type) and isinstance(old_m, comp_type)

                    if cmp(str) or cmp(int) or cmp(float) or cmp(complex):
                        return new_m

                    if cmp(dict):
                        for k in new_m.keys():
                            old_m[k] = meta_merge(new_m[k], old_m.get(k, {}), path_acc + [k])
                        return new_m

                    raise Exception("Type mismatch, cowardly refusing to proceed. at "
                                    + '.'.join(path_acc) + " Got new type: "
                                    + str(type(new_m)) + " old type: " + str(type(old_m)))
                metadata = meta_merge(metadata, old_metadata, [])
            else:
                pass # Just use the metadata we were passed

            org_info['metadata'] = metadata
            request = self.session.put(organization_url, data=json.dumps(org_info), headers=header)

            if request.status_code in [200, 201]:
                return json.loads(request.content)

            raise Exception(request.content)

        except Exception as error:
            raise error

    @single_org
    def rename_organization(self, org_id=None, *, new_name):
        """
        Rename an existing organization

        :param org: Org id to rename
        :param new_name: New name to set to the org
        :return:
        """

        header = self.build_header(
            aut_token=self.props.get_auth_token(cluster=self.cluster),
            others=self.content_type_header
        )

        organization_url = self.org_api_url.format(org_id)

        payload = {"id": org_id, "name": new_name}

        try:

            request = self.session.put(organization_url, data=json.dumps(payload), headers=header)

            if request.status_code in [200, 201]:
                return json.loads(request.content)

            raise Exception(request.content)

        except Exception as error:
            raise error

    @single_org
    def extend_expiration(self, org_id=None, *, days):
        """
        Extend organization time

        :param org_id: Org id to rename
        :param days: Days until org expires
        :return:
        """

        header = self.build_header(
            aut_token=self.props.get_auth_token(cluster=self.cluster),
            others=self.content_type_header
        )

        organization_url = self.org_api_url.format(org_id)

        payload = {"id": org_id, "expiry": days}

        try:

            request = self.session.put(organization_url, data=json.dumps(payload), headers=header)

            if request.status_code in [200, 201]:
                return json.loads(request.content)

            raise Exception(request.content)

        except Exception as error:
            raise error

    def move_user_to_group(self, user_id, group_id):
        """
        Change user to a given group
        :param user_id:
        :param group_id:
        :return:
        """

        try:

            header = self.build_header(
                aut_token=self.props.get_auth_token(cluster=self.cluster),
                others=self.different_content_type
            )

            invite_user_post = self.session.post(
                self.change_group_url.format(group_id),
                headers=header,
                data=str(user_id)
            )

            if invite_user_post.status_code in range(200, 201):
                return json.loads(invite_user_post.content)

            raise Exception(json.loads(invite_user_post.content))

        except Exception as error:

            raise error

    def retrieve_all_users(self):
        """
        Retrieves all users who the loged in user can retrieve
        :return:
        """
        try:
            all_users_list = self.simple_get(final_url=self.all_users_url)

            if all_users_list.status_code in range(200, 201):
                return json.loads(all_users_list.content)

            raise Exception(json.loads(all_users_list.content))

        except Exception as error:

            raise error

    def retrieve_all_users_form_org(self, org_id):
        """
        Retrieves all users who the loged in user can retrieve
        :return:
        """
        try:
            all_users_list = self.simple_get(final_url=self.all_users_on_org.format(org_id))

            if all_users_list.status_code in range(200, 201):
                return json.loads(all_users_list.content)

            raise Exception(json.loads(all_users_list.content))

        except Exception as error:

            raise error

    def get_all_groups(self):
        """

        :return:
        """
        try:
            all_users_list = self.simple_get(final_url=self.all_groups_url)

            if all_users_list.status_code in range(200, 201):
                return json.loads(all_users_list.content)

            raise Exception(json.loads(all_users_list.content))

        except Exception as error:

            raise error



# Copyright 2021 Element Analytics, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
from tests import *
from unify.helpers.pipeline.schema import FlowSchema


class TestSourcesWithOrgID(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.sources = Sources(
            cluster=cluster_name,
            props=props
        )
        cls.dataset_name = 'test-{}'.format(str(uuid.uuid4()))

    def test_source_list_default_org(self):
        source_list = self.sources.get_sources(
            org_id=test_org
        )

        self.assertTrue(len(source_list) > 0, source_list)

    def test_source_was_created(self):

        try:
            created_source = self.sources.download_dataset_content(
                org_id=test_org,
                dataset_id=test_dataset["id"]
            )

            self.assertTrue(True, created_source)

        except Exception as e:

            self.assertTrue(False, e)

    def test_big_dataset(self):
        big = self.sources.upload_big_dataset(
            name=str(uuid.uuid4()),
            org_id=test_org,
            content=open("tests/data_test.csv").read()
        )
        self.assertTrue("create" in big)

    def test_create_api_dataset(self):
        api = self.sources.create_api_data_set(
            org_id=test_org,
            name=str(uuid.uuid4()),
            file_path="tests/data_test.csv"
        )

        self.assertTrue("data_set_id" in api, api)

    def test_create_api_dataset_content(self):
        api = self.sources.create_api_data_set_with_content(
            name=str(uuid.uuid4()),
            org_id=test_org,
            content=open("tests/data_test.csv").read()
        )

        self.assertTrue("data_set_id" in api, api)

    def test_commit_status(self):
        api = self.sources.create_api_data_set(
            name=str(uuid.uuid4()),
            org_id=test_org,
            file_path="tests/data_test.csv"
        )

        commit, response_code = self.sources.get_commit_status(
            org_id=test_org,
            commit_id=api["commit_id"],
            data_set_id=api["data_set_id"]
        )

        self.assertTrue(response_code in [200, 202, 201], commit)

    def test_export_dataset(self):
        export = self.sources.create_export_dataset(
            org_id=test_org,
            dataset_ids=[test_dataset["id"]]
        )

        self.assertTrue(len(export) > 0, export)

    def test_append_data(self):
        export = self.sources.add_data_to_existing_source(
            org_id=test_org,
            name=str(uuid.uuid4()),
            data_set_id=test_dataset["id"],
            file_path="tests/data_test.csv"
        )

        self.assertTrue("status" in export, export)

    def test_overwrite_dataset(self):
        export = self.sources.overwrite_dataset(
            org_id=test_org,
            data_set_id=test_dataset["id"],
            file_path="tests/data_test.csv"
        )

        self.assertTrue("truncate" in export, export)
        self.assertTrue("stage" in export, export)
        self.assertTrue("append" in export, export)

    def test_truncate_data_set(self):
        export = self.sources.truncate_data_set(
            org_id=test_org,
            data_set_id=test_dataset["id"]
        )

        self.assertTrue("commit_id" in export, export)

    def test_append_data_v2(self):
        export = self.sources.append_dataset(
            data_set_id=test_dataset["id"],
            content=open("tests/data_test.csv").read(),
            org_id=test_org
        )

        self.assertTrue("id" in export, export)

    def test_static_file_upload(self):
        export = self.sources.static_file_upload(
            org_id=test_org,
            name=str(uuid.uuid4()),
            content="tests/data_test.csv"
        )

        self.assertTrue("id" in export, export)

    def test_delete_source(self):
        export = self.sources.static_file_upload(
            org_id=test_org,
            name=str(uuid.uuid4()),
            content="tests/data_test.csv"
        )

        delete = self.sources.delete_source(
            org_id=test_org,
            source_id=export["id"]
        )

        self.assertEqual(delete['message'], 'Success', delete)

    def test_rename_datset(self):
        export = self.sources.static_file_upload(
            org_id=test_org,
            name=str(uuid.uuid4()),
            content="tests/data_test.csv"
        )

        try:
            _ = self.sources.update_dataset_metadata(
                org_id=test_org,
                dataset_id=export["id"],
                name="{}__renamed".format(str(uuid.uuid4()))
            )
            self.assertTrue(True, "Pipeline was renamed")
        except Exception as e:
            self.assertTrue(False, e)

    def test_add_facets_datset(self):
        export = self.sources.static_file_upload(
            org_id=test_org,
            name=str(uuid.uuid4()),
            content="tests/data_test.csv"
        )

        try:
            _ = self.sources.update_dataset_metadata(
                org_id=test_org,
                dataset_id=export["id"],
                facets=[str(uuid.uuid4())[:10]]
            )
            self.assertTrue(True, "Facets were added to dataset")
        except Exception as e:
            self.assertTrue(False, e)

    def test_add_description_datset(self):
        export = self.sources.static_file_upload(
            org_id=test_org,
            name=str(uuid.uuid4()),
            content="tests/data_test.csv"
        )

        try:
            _ = self.sources.update_dataset_metadata(
                org_id=test_org,
                dataset_id=export["id"],
                description="{}__renamed".format(str(uuid.uuid4()))
            )
            self.assertTrue(True, "Description was added to dataset")
        except Exception as e:
            self.assertTrue(False, e)

    def test_utf_encoding_source(self):
        try:
            _ = self.sources.static_file_upload(
                org_id=test_org,
                name=str(uuid.uuid4()),
                content="tests/data_test.csv",
                encoding="UTF-8"

            )
            self.assertTrue(True, "Dataset was created")
        except Exception as e:
            self.assertTrue(False, e)

    def test_wait_for_pipeline(self):
        """
        Verify waitfor dataset endpoint
        :return:
        """
        try:
            _ = self.sources.wait_for_dataset(
                dataset_id=test_dataset["id"],
                org_id=test_org
            )

            self.assertTrue(True, "Dataset became ready")
        except Exception as e:
            self.assertTrue(False, e)

    def test_create_empty_dataset(self):
        """
        Verify empty dataset can be created
        :return:
        """

        try:
            response = self.sources.create_command(
                org_id=test_org,
                schema={
                    "columns": [
                        {
                            "header": "name",
                            "column": {
                                "type": "text",
                                "properties": {
                                    "optional": False,
                                    "datasets.v1.columnType": "Normal"
                                }
                            }
                        },
                        {
                            "header": "description",
                            "column": {
                                "type": "text",
                                "properties": {
                                    "optional": False,
                                    "datasets.v1.columnType": "Normal"
                                }
                            }
                        }
                    ],
                    "properties": {}
                },
                name=str(uuid.uuid4()),
                facets=[],
                description="This is a test",
                cause=[]
            )

            self.assertTrue(response["uuid"] is not None, "Dataset was created")

        except Exception as e:

            self.assertTrue(False, e)

    def test_create_empty_dataset_from_flow(self):

        flow_cols = [
            {
                "name": "name",
                "dataType": "STRING",
                "isOpt": True,
                "columnType": "Normal"
            },
            {
                "name": "description",
                "dataType": "STRING",
                "isOpt": True,
                "columnType": "Normal"
            }, {
                "name": "engunits",
                "dataType": "STRING",
                "isOpt": True,
                "columnType": "UnitOfMeasure"
            }, {
                "name": "EQUIPMENT_ID *",
                "dataType": "STRING",
                "isOpt": False,
                "columnType": "EquipmentId"
            }
        ]

        schema = {
            "columns": FlowSchema(columns=flow_cols).to_dataset_schema(),
            "properties": {}
        }

        try:
            response = self.sources.create_command(
                org_id=test_org,
                schema=schema,
                name=str(uuid.uuid4()),
                facets=[],
                description="This is a test",
                cause=[]
            )

            self.assertTrue(response["uuid"] is not None, "Dataset from flow created")

        except Exception as e:

            self.assertTrue(False, e)



# Copyright 2021 Element Analytics, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Contains methods to interact with connector service
"""
import json
from enum import Enum
from datetime import datetime

from unify.properties import Properties
from unify.properties import ClusterSetting
from unify.apirequestsmng import ApiRequestManager
from unify.helpers.SingleOrg import single_org

class ConnStatus(str, Enum):
    """
    Connector status
    """
    ERROR = 'error'
    GOOD = 'good'
    UNKNOWN = 'unknown'

class LogType(str, Enum):
    """
    Log message severity
    """
    ERROR = 'error'
    INFO = 'info'
    SUCCESS = 'success'
    WARNING = 'warning'

class ArtifactType(str, Enum):
    """
    Artifact type
    """
    CONNECTOR = 'connector'
    DATASET = 'dataset'
    GRAPH = 'graph'
    PIPELINE = 'pipeline'
    TEMPLATE = 'template'


class Connectors(ApiRequestManager):
    """
    Class to interact with connectors endpoints
    """

    def __init__(self, cluster=None, props=Properties(ClusterSetting.KEY_RING), org_id=None):

        """
        Class constructor

        :param cluster: Cluster name to be used
        :type cluster: str
        :param props: Properties instantiated object
        :type props: class:`unify.properties.Properties`
        """
        super().__init__(cluster=cluster, props=props, org_id=org_id)

        remote = self.props.get_remote(self.cluster)

        self.connectors_url = remote + 'connectors/'
        self.update_status_url = self.connectors_url + "{}/updateStatus"
        self.update_config_url = self.connectors_url + "{}/updateConfig"
        self.get_connector_url = self.connectors_url + "{}"
        self.write_log_url = self.connectors_url + "{}/writeLog"
        self.get_artifacts_url = self.connectors_url + "{}/artifacts"
        self.healthcheck_url = self.connectors_url + "healthcheck"
        self.create_with_user_url = self.connectors_url + "create-with-user"

    @single_org
    def create_connector(self, org_id=None, *, name, connector_type, config):
        """
        Create a new connector

        :param org_id: organization Id to be queried
        :param name: connector name
        :param connector_type: type of the connector
        :param config: connector config
        :return: connector id
        """
        header = self.build_header(
            org_id=org_id,
            aut_token=self.props.get_auth_token(cluster=self.cluster),
            others=self.content_type_header
        )

        body = {
            'name': name,
            'type': connector_type,
            'config': config,
        }

        result = self.session.post(self.connectors_url, headers=header, json=body)

        if result.status_code in self.OK:
            return json.loads(result.content)

        raise Exception(repr(result.content))

    @single_org
    def update_connector_status(self, org_id=None, *, connector_id, status, metadata=''):
        """
        Updates connector status

        :param org_id: organization Id to be queried
        :param connector_id: Connector id to be updated
        :param status: new status to update
        :param metadata: connector metadata to be updated
        :return: API response
        """

        header = self.build_header(
            org_id=org_id,
            aut_token=self.props.get_auth_token(cluster=self.cluster),
            others=self.content_type_header
        )

        body = {
            'status': status,
            'metadata': metadata
        }

        result = self.session.put(
            self.update_status_url.format(connector_id),
            headers=header,
            json=body
        )

        if result.status_code in self.OK:
            return True

        raise Exception(repr(result.content))

    @single_org
    def update_connector_config(self, org_id=None, *, connector_id, config):
        """
        Update connector configuration

        :param org_id: organization Id to be queried
        :param connector_id: Connector id to be updated
        :param config: Config to be updated
        """
        header = self.build_header(
            org_id=org_id,
            aut_token=self.props.get_auth_token(cluster=self.cluster)
        )
        result =  self.session.put(
            self.update_config_url.format(connector_id),
            headers=header,
            json=config
        )

        if result.status_code in self.OK:
            return True

        raise Exception(repr(result.content))

    @single_org
    def get_connector(self, org_id=None, *, connector_id):
        """
        Returns the description of a given connector. Will be called periodically by the connector
        to get updated configuration

        :param org_id: organization Id to be queried
        :param connector_id: Connector id to be returned
        :return: Connector info
        """
        header = self.build_header(
            org_id=org_id,
            aut_token=self.props.get_auth_token(cluster=self.cluster)
        )
        result = self.session.get(self.get_connector_url.format(connector_id), headers=header)

        if result.status_code in self.OK:
            return json.loads(result.content)

        raise Exception(repr(result.content))

    @single_org
    def delete_connector(self, org_id=None, *, connector_id):
        """
        Delete connector

        :param org_id: organization Id to be queried
        :param connector_id: Connector id to be returned
        """
        header = self.build_header(
            org_id=org_id,
            aut_token=self.props.get_auth_token(cluster=self.cluster)
        )
        result = self.session.delete(self.get_connector_url.format(connector_id), headers=header)

        if result.status_code in self.OK:
            return True

        raise Exception(repr(result.content))

    @single_org
    def write_log(
        self,
        org_id=None,
        *,
        connector_id,
        message,
        timestamp=None,
        log_type=LogType.INFO,
        metadata=''
    ):
        """
        Add a log entry

        :param org_id: organization Id to be queried
        :param connector_id: Connector id to be returned
        :param message: log message
        :param timestamp: timestamp of the log message
        :param type: severity of the log
        :param metadata: log message metadata
        """
        if timestamp is None:
            timestamp = int(datetime.now().timestamp() * 1000)

        body = [{
            "timestamp": timestamp,
            "type": log_type,
            "message": message,
            "metadata": metadata
        }]

        self.write_logs(org_id, connector_id=connector_id, log_list=body)

    @single_org
    def write_logs(self, org_id=None, *, connector_id, log_list):
        """
        Add a list of log entries

        :param org_id: organization Id to be queried
        :param connector_id: Connector id to be returned
        :param log_list: log messages in list of dicts with keys: "timestamp", "type", "message",
        "metadata"
        """

        header = self.build_header(
            org_id=org_id,
            aut_token=self.props.get_auth_token(cluster=self.cluster)
        )

        result = self.session.post(
            self.write_log_url.format(connector_id),
            headers=header,
            json=log_list
        )

        if result.status_code in self.OK:
            return True

        raise Exception(repr(result.content))

    @single_org
    def get_artifacts(self, org_id=None, *, connector_id):
        """
        Get registered artifacts

        :param org_id: organization Id to be queried
        :param connector_id: Connector id to be returned
        :return: List of artifacts linked
        """
        header = self.build_header(
            org_id=org_id,
            aut_token=self.props.get_auth_token(cluster=self.cluster)
        )
        result = self.session.get(self.get_artifacts_url.format(connector_id), headers=header)

        if result.status_code in self.OK:
            return json.loads(result.content)

        raise Exception(repr(result.content))

    @single_org
    def create_artifacts(self, org_id=None, *, connector_id, artifacts_list):
        """
        Register artifacts

        :param org_id: organization Id to be queried
        :param connector_id: Connector id to be returned
        :param artifacts_list: artifacts info into list of dicts with keys: "artifactType",
        "artifactId", "artifactName"
        """
        header = self.build_header(
            org_id=org_id,
            aut_token=self.props.get_auth_token(cluster=self.cluster)
        )
        result = self.session.post(
            self.get_artifacts_url.format(connector_id),
            headers=header,
            json=artifacts_list
        )
        if result.status_code in self.OK:
            return True

        raise Exception(repr(result.content))

    @single_org
    def create_artifact(
        self,
        org_id=None,
        *,
        connector_id,
        artifact_type,
        artifact_id,
        artifact_name
    ):
        """
        Register a single artifact

        :param org_id: organization Id to be queried
        :param connector_id: Connector id to be returned
        :param artifacts_list: artifacts info into list of dicts with keys: "artifactType",
        "artifactId", "artifactName"
        """
        body = [{
            "artifactType": artifact_type,
            "artifactId": artifact_id,
            "artifactName": artifact_name
        }]
        return self.create_artifacts(org_id, connector_id=connector_id, artifacts_list=body)

    def healthcheck(self, encoding='utf8'):
        """
        Returns the health status of the connector service
        """
        result = self.session.get(self.healthcheck_url)
        if result.status_code in self.OK:
            return result.content.decode(encoding)

        raise Exception(repr(result.content))

    @single_org
    def create_connector_with_user(self, org_id=None, *, name, connector_type, config):
        """
        Create a new connector, creating an associated user as well

        :param org_id: organization Id to be queried
        :param name: connector name
        :param connector_type: type of the connector
        :param config: connector config
        :return: connector id and user info
        """
        header = self.build_header(
            org_id=org_id,
            aut_token=self.props.get_auth_token(cluster=self.cluster)
        )

        body = {
            'name': name,
            'type': connector_type,
            'config': config,
        }
        result = self.session.post(self.create_with_user_url, headers=header, json=body)
        if result.status_code in self.OK:
            return json.loads(result.content)

        raise Exception(repr(result.content))



import inspect


def single_org(method):
    signature = inspect.signature(method)

    def wrapper(self, *args, **kwargs):

        bound = signature.bind(*((self,) + args), **kwargs)

        bound.apply_defaults()

        k = "org_id"

        if k in bound.arguments:
            v = bound.arguments[k]
            bound.arguments.update({k: v if v else getattr(self, k)})

        return method(*bound.args, **bound.kwargs)

    return wrapper



# Copyright 2021 Element Analytics, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
from tests import *
from unify.connectors import ConnStatus, ArtifactType

class TestSourcesWithOrgID(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.connectors = Connectors(cluster=cluster_name, props=props)

    @unittest.skip('Connector service is broken')
    def test_creation_delete_connector(self):
        conn_id = self.connectors.create_connector(
            org_id=test_org,
            name='wmorales_test',
            connector_type='wmorales_type',
            config={'a':'b','c':'d'}
        )
        self.connectors.get_connector(org_id=test_org, connector_id=conn_id)
        self.connectors.delete_connector(org_id=test_org, connector_id=conn_id)
        try:
            self.connectors.get_connector(org_id=test_org, connector_id=conn_id)
            raise Exception('Connector is not deleted')
        except Exception as err:
            self.assertTrue(
                (err.args[0] == "b'The requested resource could not be found.'") or
                (err.args[0] == "b'java.util.NoSuchElementException: Invoker.first'")
            )

    @unittest.skip('Connector service is broken')
    def test_get_connector(self):
        connector = self.connectors.get_connector(org_id=test_org, connector_id=test_connector)
        result = {}
        result['id'] = connector['id']
        result['type'] = connector['type']
        self.assertEqual(result, {'id':test_connector, 'type':'connector_test_type'})

    @unittest.skip('Connector service is broken')
    def test_update_connector_status(self):
        self.connectors.update_connector_status(
            org_id=test_org,
            connector_id=test_connector,
            status=ConnStatus.GOOD,
        )
        connector = self.connectors.get_connector(org_id=test_org, connector_id=test_connector)
        self.assertEqual(connector['status'], 'good')

    @unittest.skip('Connector service is broken')
    def test_update_connector_config(self):
        self.connectors.update_connector_config(
            org_id=test_org,
            connector_id=test_connector,
            config={'e':'f','g':'h'}
        )
        connector = self.connectors.get_connector(org_id=test_org, connector_id=test_connector)
        self.assertEqual(connector['config'], {'e':'f','g':'h'})

    # TODO Run test again when endpoint is fixed
    @unittest.skip('Write logs endpoint needs to be fixed in prod')
    def test_write_log(self):
        self.connectors.write_log(
            org_id=test_org,
            connector_id=test_connector,
            message='Python SDK connectors test'
        )
        self.assertTrue(True)

    @unittest.skip('Connector service is broken')
    def test_create_get_artifact(self):
        self.connectors.create_artifact(
            org_id=test_org,
            connector_id=test_connector,
            artifact_type=ArtifactType.DATASET,
            artifact_id='12345',
            artifact_name="Python_SDK_connectors_artifact_test"
        )
        artifact = self.connectors.get_artifacts(org_id=test_org, connector_id=test_connector)
        self.assertTrue({
            'artifactType': 'dataset',
            'artifactId': '12345',
            'artifactName': 'Python_SDK_connectors_artifact_test'
        } in artifact)
